
;;  "Hobbit" wersja polska
;;  Copyright (C) 2005 Pawe Aleksander Fedoryski <pfedor@fuw.edu.pl>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(defparameter *line-width* 42)
(defparameter *indent-step* #(4 2))

(defvar *indent* 0)

(defun opis-wnetrza (przid)
  (let ((opwn (assoc (zob :opis_wnetrza przid)
                     *opisy-wnetrza* :test #'equalp)))
    (format nil (cadr opwn) (przyp (cddr opwn) przid :bez-przymiotnikow t)))) 

(defun przedmiot-info (przid &key ((:przyp przypadek)))
  (let ((zawartosc ())
        (lprz (przedmioty-wewnatrz przid))
        (los (postacie-wewnatrz przid)))
    (when (eql przid (zob :gdzie_prz *id-gracz*))
      (push *id-gracz* los))
    (setf los (remove (kto-pierscien) los))
    (when (zob :otwarte przid)
      (setf zawartosc 
            (append 
             (mapcan #'(lambda (id) (przedmiot-info id :przyp :mian)) lprz)
             (mapcan #'(lambda (id) (postac-info id :przyp :mian)) los))))
    (if (null zawartosc)
        (if przypadek (list (przyp przypadek przid)))
        (list (format nil
                      "~@[~a. ~]~a ~[znajdujesz~;znajduj~;znajduje~] si"
                      (if przypadek (przyp przypadek przid))
                      (opis-wnetrza przid)
                      (let ((f (car (append lprz los))))
                        (cond ((eql f *id-gracz*) 0)
                              ((eql :nmos (rodzaj f)) 1)
                              (t 2))))
              zawartosc))))

(defun postac-info (osid &key inventory ((:przyp przypadek)))
  (if (and (eql osid *id-gracz*)
           (not inventory))
      (when przypadek
        (list (get-const *you-long* przypadek)))
      (let* ((lprz (przedmioty-trzyma osid))
             (los (remove (kto-pierscien)
                          (append
                           (when (eql osid (zob :gdzie_osoba *id-gracz*))
                             (list *id-gracz*))
                           (postacie-trzyma osid))))
             (zawartosc
              (append
               (mapcan #'(lambda (id) (przedmiot-info id :przyp :bier)) lprz)
               (mapcan #'(lambda (id) (postac-info id :przyp :bier)) los))))
        (if (null zawartosc)
            (if przypadek (list (przyp przypadek osid)))
            (list (if (eql osid *id-gracz*)
                      "Trzymasz"
                      (format nil "~@[~a. ~]~a trzyma"
                              (if przypadek (przyp przypadek osid))
                              (przyp :mian osid
                                     :bez-przymiotnikow t
                                     :poczatek-zdania t)))
                  zawartosc)))))

(defun visible-exits-str (lok)
  (format nil "~@[Widoczne wyjcia: ~{~a~^, ~}.~]"
          (mapcar #'(lambda (k) (get-const *kierunki* k))
                  (remove-if-not #'(lambda (k) (getf lok k))
                                 (keys *kierunki*)))))

(defun door-str (k rid)
  (concatenate 'string
               "Na " (get-const *kierdrzwi* k)
               (if (eql (rodzaj rid) :nmos)
                   " znajduj si "
                   " znajduje si ")
               (przyp :mian rid) "."))

(defun you-see-list (id)
  (let* ((tt (find-type id))
         (los (ecase tt
                (:mapa (remove (kto-pierscien) (postacie-w-lokacji id)))
                (:przedmioty (remove (kto-pierscien)
                                     (postacie-wewnatrz id)))))
         (lprz (ecase tt
                 (:mapa (przedmioty-w-lokacji id))
                 (:przedmioty (przedmioty-wewnatrz id)))))
    (if (and (null los) (null lprz))
        '("Nic interesujcego.")
        (append
         (mapcan #'(lambda (przid) (przedmiot-info przid :przyp :bier)) lprz)
         (mapcan #'(lambda (osid) (postac-info osid :przyp :bier)) los)))))

(defun desc-location (lokid &key short)
  (let ((lok (find-object lokid))
        (retv '())
        rid)
    (push (getf lok (if short :krotkiop :dlugiop)) retv)
    (when (and (not short) (getf lok :anim))
      (push `(:animation ,(getf lok :anim)) retv))
    (when (not short)
      (dolist (k (keys *kierdrzwi*))
	(when (setf rid (getf lok k))
	  (push (door-str k rid) retv))))
    (let ((ves (visible-exits-str lok)))
      (when (> (length ves) 0)
        (push ves retv)))
    (push "Widzisz:" retv)
    (push (you-see-list lokid) retv)
    (nreverse retv)))

(defun opisz-przedmiot (przid)
  (list (format nil "Jeste w ~a."
                (przyp :miejsc przid :bez-przymiotnikow t))
        "Widzisz:"
        (you-see-list przid)))

(defun print-str (s)
  (loop with av = (- *line-width* *indent*) and used = 0 and w do
       (setf (values w s) (get-one-word s))
       (when (null w)
         (format t "~%")
         (return))
       (if (equal w "%")
           (progn
             (format t "~%")
             (setf used 0))
           (progn
             (when (and (> (1+ (+ used (length w))) av)
                        (not (eql 0 used)))
               (format t "~%")
               (setf used 0))
             (when (eql 0 used)
               (dotimes (i *indent*) (format t " ")))
             (unless (eql 0 used)
               (format t " ")
               (setf used (1+ used)))
             (format t "~a" w)
             (setf used (+ used (length w)))))))

(defun animation (anim)
  (format t "[ANIMATION]~a~%" anim))

(defun print-description-int (desc)
  (dolist (el desc)
    (cond 
      ((atom el)
       (print-str el))
      ((eql (car el) :animation)
       (animation (cadr el)))
      (t
       (let ((*indent* (+ *indent* (if (eql 0 *indent*)
                                       (elt *indent-step* 0)
                                       (elt *indent-step* 1)))))
         (print-description-int el))))))

(defun print-description (desc)
  (let ((po-ciemku nil))
    (when (and (not (null desc))
               (eql (car desc) :ciemno))
      (pop desc)
      (setf po-ciemku t))
    (when (or po-ciemku (not (czy-jest-ciemno)))
      (print-description-int desc))))



