
;;  "Hobbit" wersja polska
;;  Copyright (C) 2005 Pawe Aleksander Fedoryski <pfedor@fuw.edu.pl>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(defun lfn (&rest args)
  (list (apply #'format (list* nil args))))

(defun ciemno-lfn (&rest args)
  (append '(:ciemno) (apply #'lfn args)))

(defun report (osid niewidzialny &optional gracz grlok &rest inne)
  (if (eql osid *id-gracz*)
      gracz
      (when (or (not (eql (kto-pierscien) osid))
                niewidzialny)
        (let ((grl (gdzie *id-gracz*)))
          (if (eql grl (gdzie osid))
              grlok
              (loop for l on inne by #'cddr do
                    (when (find grl (car l))
                      (return (cadr l)))))))))

(defun find-door (lokid direction)
  (let ((dd (intern (concatenate 'string (symbol-name direction) "D")
                    "KEYWORD")))
    (zob dd lokid)))

(defun explicit-door-destination (lokid direction)
  (let ((dd-> (intern (concatenate 'string (symbol-name direction) "D->")
                      "KEYWORD")))
    (zob dd-> lokid)))

(defun find-destination (lokid direction)
  (or (zob direction lokid)
      (explicit-door-destination lokid direction)
      (let ((drzid (find-door lokid direction)))
        (when (and drzid
                   (or (zob :otwarte drzid)
                       (zob :rozbite drzid)))
          (car (remove lokid (lokacje-z-drzwiami drzid)))))))

(defun nie-dosiegnie (osid drzwi)
  (when (or (and (eql osid *id-gracz*)
                 (eql drzwi (get-label-id "window")))
            (and (eql (gdzie osid) (get-label-id "forestriver2"))
                 (eql drzwi (get-label-id "ltrapdoor"))))
    (format nil "Nie dosigniesz do ~a." (dop drzwi))))

(defun go-direction-ok (osid &key direction)
  (unless (and (eql (find-type (gdzie osid :stop :przedmiot)) :mapa)
               (or (zob direction (gdzie osid))
                   (let ((drzid (find-door (gdzie osid) direction)))
                     (and drzid
                          (not (nie-dosiegnie osid drzid))
                          (or (zob :otwarte drzid)
                              (zob :rozbite drzid))))))
    (format nil "Nie moesz i ~a." (get-const *kierunki2* direction))))

(defun go-direction (osid &key direction)
  (let (idold
        idnew
        (bylo-ciemno (czy-jest-ciemno))
        (descloc '()))
    (setf idold (gdzie osid))
    (setf idnew (find-destination idold direction))
    (let ((retv
           (if (eql osid *id-gracz*)
               (let ((sh (zob :visited idnew)))
                 (setf (zob :visited idnew) t)
                 (when (not sh)
                   (setf *score* (+ *score* (zob :score idnew))))
                 (setf descloc (desc-location idnew :short sh))
                 (ciemno-lfn "Idziesz ~a."
                             (if bylo-ciemno
                                 "w losowym kierunku"
                                 (get-const *kierunki2* direction))))
               (when (not (eql (kto-pierscien) osid))
                 (cond
                   ((eql idold (gdzie *id-gracz*))
                    (funcall (if bylo-ciemno #'lfn #'ciemno-lfn)
                             "~a idzie ~a."
                             (mian osid :poczatek-zdania t)
                             (get-const *kierunki2* direction)))
                   ((eql idnew (gdzie *id-gracz*))
                    (ciemno-lfn "Wchodzi ~a." (mian osid))))))))
      (setf (gdzie osid) idnew)
      (when (and (eql osid *id-gracz*)
                 (not (czy-jest-ciemno)))
        (setf retv (append retv descloc)))
      retv)))

(defun gk (id fmt)
  (let ((m (mian id :poczatek-zdania t :bez-przymiotnikow t))
        (ec1 (ecase (rodzaj id)
               (:nmos 0)
               ((:m :z :n) 1)))
        (ec2 (ecase (rodzaj id)
               ((:nmos :n) 0)
               (:m 1)
               (:z 2))))
    (format nil fmt m ec1 ec2)))

(defun ile-wazy (obj)
  (+ (or (zob :ciezar obj) 0)
     (if (eql obj (or (zob :gdzie_prz *id-gracz*)
                      (zob :gdzie_osoba *id-gracz*)))
         (ile-wazy *id-gracz*)
         0)
     (reduce
      #'+
      (mapcar #'ile-wazy 
              (if (eql (find-type obj) :przedmioty)
                  (append (przedmioty-wewnatrz obj) (postacie-wewnatrz obj))
                  (append (przedmioty-trzyma obj) (postacie-trzyma obj)))))))

(defun nie-podniesie (osid obj)
  (cond
    ((eql (gdzie osid :stop obj) obj)
     (if (eql (find-type obj) :przedmioty)
         (format nil "Jeste w rodku ~a." (dop obj))
         (format nil "~a ci niesie." (mian obj :poczatek-zdania t))))
    ((> (ile-wazy obj) (or (zob :sila osid) 0))
     (gk obj "~a ~[s~;jest~] za ~[cikie~;ciki~;cika~]."))
    ((eql obj (get-label-id "wine"))
     "To niemoliwe.")
    (t nil)))

(defun wait-ok (osid)
  (declare (ignore osid))
  nil)

(defun wait (osid)
  (report osid nil
          (list "Czekasz." "Czas pynie...")
          (lfn "~a czeka." (mian osid :poczatek-zdania t))))

(defun look-ok (osid)
  (declare (ignore osid))
  nil)

(defun look (osid)
  (report osid nil
          (let ((id-m (gdzie osid :stop :prz)))
            (ecase (find-type id-m)
              (:mapa (desc-location id-m))
              (:przedmioty (opisz-przedmiot id-m))))
          (lfn "~a rozglda si dokoa." (mian osid :poczatek-zdania t))))

(defun zlamane-info (prz)
  (format nil "~a ~:[jest~;s~] ~a."
          (mian prz :bez-przymiotnikow t :poczatek-zdania t)
          (eql (rodzaj prz) :nmos)
          (zob :mian (zob :zlamane_przym prz) (rodzaj prz))))

(defun rozbite-info (drzwi)
  (format nil "~a ~:[jest~;s~] ~a."
          (mian drzwi :bez-przymiotnikow t :poczatek-zdania t)
          (eql (rodzaj drzwi) :nmos)
          (zob :mian (zob :rozbite_przym drzwi) (rodzaj drzwi))))

(defun ze-zlej-strony (osid drzwi)
  (when (eql (gdzie osid) (zob :zla_strona drzwi))
    (format nil "Nie moesz otworzy ~a." (dop drzwi))))

(defun otwdrzwi-ok (osid &key drzwi)
  (cond
    ((not (zob :otwieralne drzwi))
     (format nil "Nie moesz otworzy ~a."
             (dop drzwi :bez-przymiotnikow t)))
    ((zob :rozbite drzwi)
     (rozbite-info drzwi))
    ((zob :otwarte drzwi)
     (gk drzwi "~a ~[s~;jest~] ~[otwarte~;otwarty~;otwarta~]."))
    ((zob :locked drzwi)
     (gk drzwi
         "~a ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~] na klucz."))
    ((nie-dosiegnie osid drzwi))
    ((ze-zlej-strony osid drzwi))
    (t nil)))

(defun otwdrzwi (osid &key drzwi)
  (setf (zob :otwarte drzwi) t)
  (report osid t
          (lfn "Otwierasz ~a." (bier drzwi))
          (lfn "~a otwiera ~a." (mian osid :poczatek-zdania t) (bier drzwi))
          (lokacje-z-drzwiami drzwi)
          (lfn "Kto otwiera ~a." (bier drzwi))))

(defun zamkdrz-ok (osid &key drzwi)
  (cond
    ((not (zob :otwieralne drzwi))
     (format nil "Nie moesz zamkn ~a."
             (dop drzwi :bez-przymiotnikow t)))
    ((zob :rozbite drzwi)
     (rozbite-info drzwi))
    ((not (zob :otwarte drzwi))
     (gk drzwi "~a ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~]."))
    ((nie-dosiegnie osid drzwi))
    (t nil)))

(defun zamkdrz (osid &key drzwi)
  (setf (zob :otwarte drzwi) nil)
  (report osid t
          (lfn "Zamykasz ~a." (bier drzwi))
          (lfn "~a zamyka ~a." (mian osid :poczatek-zdania t) (bier drzwi))
          (lokacje-z-drzwiami drzwi)
          (lfn "Kto zamyka ~a." (bier drzwi))))

(defun otwdrzkl-ok (osid &key drzwi klucz)
  (cond
    ((not (zob :otwieralne drzwi))
     (format nil "Nie moesz otworzy ~a."
             (dop drzwi :bez-przymiotnikow t)))
    ((zob :rozbite drzwi)
     (rozbite-info drzwi))
    ((zob :otwarte drzwi)
     (gk drzwi "~a ~[s~;jest~] ~[otwarte~;otwarty~;otwarta~]."))
    ((nie-podniesie osid klucz))
    ((nie-dosiegnie osid drzwi))
    ((not (zob :locked drzwi))
     (gk drzwi
         "~a nie ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~] na klucz."))
    ((not (eql klucz (zob :klucz drzwi)))
     (format nil "~a nie ~[pasuj~;pasuje~] do ~a."
             (mian klucz :poczatek-zdania t)
             (ecase (rodzaj klucz)
               (:nmos 0)
               ((:m :z :n) 1))
             (dop drzwi)))
    ((zob :zlamane klucz)
     (zlamane-info klucz))
    (t nil)))

(defun otwdrzkl (osid &key drzwi klucz)
  (setf (zob :otwarte drzwi) t (zob :locked drzwi) nil)
  (report osid t
          (lfn "Otwierasz ~a ~a." (bier drzwi) (narz klucz))
          (lfn "~a otwiera ~a ~a." (mian osid :poczatek-zdania t)
               (bier drzwi) (narz klucz))
          (lokacje-z-drzwiami drzwi)
          (lfn "Kto otwiera ~a." (bier drzwi))))

(defun zamkdrzkl-ok (osid &key drzwi klucz)
  (cond
    ((not (zob :otwieralne drzwi))
     (format nil "Nie moesz zamkn ~a."
             (dop drzwi :bez-przymiotnikow t)))
    ((zob :rozbite drzwi)
     (rozbite-info drzwi))
    ((zob :locked drzwi)
     (gk drzwi
         "~a ~[s~;jest~] ju ~[zamknite~;zamknity~;zamknita~] na klucz."))
    ((nie-podniesie osid klucz))
    ((nie-dosiegnie osid drzwi))
    ((not (eql klucz (zob :klucz drzwi)))
     (format nil "~a nie ~[pasuj~;pasuje~] do ~a."
             (mian klucz :poczatek-zdania t)
             (ecase (rodzaj klucz)
               (:nmos 0)
               ((:m :z :n) 1))
             (dop drzwi)))
    ((zob :zlamane klucz)
     (zlamane-info klucz))
    (t nil)))

(defun zamkdrzkl (osid &key drzwi klucz)
  (setf (zob :otwarte drzwi) nil (zob :locked drzwi) t)
  (report osid t
          (lfn "Zamykasz ~a ~a." (bier drzwi) (narz klucz))
          (lfn "~a zamyka ~a ~a." (mian osid :poczatek-zdania t)
               (bier drzwi) (narz klucz))
          (lokacje-z-drzwiami drzwi)
          (lfn "Kto zamyka ~a na klucz." (bier drzwi))))

(defun otwprz-ok (osid &key przedmiot)
  (declare (ignore osid))
  (cond
    ((not (zob :otwieralne przedmiot))
     (format nil "Nie moesz otworzy ~a."
             (dop przedmiot :bez-przymiotnikow t)))
    ((zob :zlamane przedmiot)
     (zlamane-info przedmiot))
    ((zob :otwarte przedmiot)
     (gk przedmiot "~a ~[s~;jest~] ~[otwarte~;otwarty~;otwarta~]."))
    ((zob :locked przedmiot)
     (gk przedmiot
         "~a ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~] na klucz."))
    (t nil)))

(defun otwprz (osid &key przedmiot)
  (setf (zob :otwarte przedmiot) t)
  (destructuring-bind (opis kto) 
      (if (eql (gdzie *id-gracz* :stop przedmiot) przedmiot)
          (list () "Kto")
          (list (przedmiot-info przedmiot)
                (mian osid :poczatek-zdania t)))
    (report osid t
            (list* (format nil "Otwierasz ~a." (bier przedmiot))
                   opis)
            (list* (format nil "~a otwiera ~a." kto (bier przedmiot))
                   opis))))

(defun zamkprz-ok (osid &key przedmiot)
  (declare (ignore osid))
  (cond
    ((not (zob :otwieralne przedmiot))
     (format nil "Nie moesz zamkn ~a."
             (dop przedmiot :bez-przymiotnikow t)))
    ((zob :zlamane przedmiot)
     (zlamane-info przedmiot))
    ((not (zob :otwarte przedmiot))
     (gk przedmiot "~a ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~]."))
    (t nil)))

(defun zamkprz (osid &key przedmiot)
  (let ((retv (report osid t
                      (lfn "Zamykasz ~a." (bier przedmiot))
                      (lfn "~a zamyka ~a."
                           (mian osid :poczatek-zdania t)
                           (bier przedmiot)))))
    (setf (zob :otwarte przedmiot) nil)
    retv))

(defun otwprzkl-ok (osid &key przedmiot klucz)
  (cond
    ((not (zob :otwieralne przedmiot))
     (format nil "Nie moesz otworzy ~a."
             (dop przedmiot :bez-przymiotnikow t)))
    ((zob :zlamane przedmiot)
     (zlamane-info przedmiot))
    ((zob :otwarte przedmiot)
     (gk przedmiot "~a ~[s~;jest~] ~[otwarte~;otwarty~;otwarta~]."))
    ((not (zob :locked przedmiot))
     (gk przedmiot
         "~a nie ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~] na klucz."))
    ((nie-podniesie osid klucz))
    ((not (eql klucz (zob :klucz przedmiot)))
     (format nil "~a nie ~[pasuj~;pasuje~] do ~a."
             (mian klucz :poczatek-zdania t)
             (ecase (rodzaj klucz)
               (:nmos 0)
               ((:m :z :n) 1))
             (dop przedmiot)))
    ((zob :zlamane klucz)
     (zlamane-info klucz))
    (t nil)))

(defun otwprzkl (osid &key przedmiot klucz)
  (setf (zob :otwarte przedmiot) t (zob :locked przedmiot) nil)
  (destructuring-bind (opis kto kl)
      (if (eql (gdzie *id-gracz* :stop przedmiot) przedmiot)
          (list () "Kto" nil)
          (list (przedmiot-info przedmiot)
                (mian osid :poczatek-zdania t)
                (narz klucz)))
    (report osid t
            (list* (format nil "Otwierasz ~a ~a."
                           (bier przedmiot) (narz klucz))
                   opis)
            (list* (format nil "~a otwiera ~a~@[ ~a~]."
                           kto (bier przedmiot) kl)
                   opis))))

(defun zamkprzkl-ok (osid &key przedmiot klucz)
  (cond
    ((not (zob :otwieralne przedmiot))
     (format nil "Nie moesz zamkn ~a."
             (dop przedmiot :bez-przymiotnikow t)))
    ((zob :zlamane przedmiot)
     (zlamane-info przedmiot))
    ((zob :locked przedmiot)
     (gk przedmiot
         "~a ~[s~;jest~] ju ~[zamknite~;zamknity~;zamknita~] na klucz."))
    ((nie-podniesie osid klucz))
    ((not (eql (zob :klucz przedmiot) klucz))
     (format nil "~a nie ~[pasuj~;pasuje~] do ~a."
             (mian klucz :poczatek-zdania t)
             (ecase (rodzaj klucz)
               (:nmos 0)
               ((:m :z :n) 1))
             (dop przedmiot)))
    ((zob :zlamane klucz)
     (zlamane-info klucz))
    (t nil)))

(defun zamkprzkl (osid &key przedmiot klucz)
  (let ((retv (report osid t
                      (lfn "Zamykasz ~a ~a." (bier przedmiot) (narz klucz))
                      (lfn "~a zamyka ~a ~a." (mian osid :poczatek-zdania t)
                           (bier przedmiot) (narz klucz)))))
    (setf (zob :otwarte przedmiot) nil (zob :locked przedmiot) t)
    retv))

(defun putinto-ok (osid &key co doczego)
  (cond
    ((eql co doczego)
     (format
      nil "Nie moesz woy ~a do ~[nich samych~;niego samego~;niej samej~]."
      (dop co)
      (ecase (rodzaj co)
        (:nmos 0)
        ((:m :n) 1)
        (:z 2))))
    ((not (zob :z_wnetrzem doczego))
     (format nil "Do ~a nie da si nic woy." (dop doczego)))
    ((and (zob :otwieralne doczego) (not (zob :otwarte doczego)))
     (gk doczego "~a ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~]."))
    ((eql co (gdzie doczego :stop co))
     (format nil "~a ~[s~;jest~] w rodku ~a."
             (mian doczego :poczatek-zdania t)
             (ecase (rodzaj doczego)
               (:nmos 0)
               ((:m :z :n) 1))
             (dop co)))
    ((eql doczego (zob :gdzie_prz co))
     (format nil "~a ju ~[s~;jest~] w ~a." (mian co :poczatek-zdania t)
             (ecase (rodzaj co)
               (:nmos 0)
               ((:m :z :n) 1))
             (miejsc doczego)))
    ((nie-podniesie osid co))
    (t nil)))

(defun putinto (osid &key co doczego)
  (setf (gdzie co) doczego)
  (report osid t
          (lfn "Wkadasz ~a do ~a." (bier co) (dop doczego))
          (lfn "~a wkada ~a do ~a." (mian osid :poczatek-zdania t)
               (bier co) (dop doczego))))

(defun wyjmij-ok (osid &key co zczego)
  (cond
    ((not (eql zczego (gdzie co :stop zczego)))
     (format nil "~a nie ~[s~;jest~] w ~a." (mian co :poczatek-zdania t)
             (ecase (rodzaj co)
               (:nmos 0)
               ((:m :z :n) 1))
             (miejsc zczego)))
    ((nie-podniesie osid co))
    (t nil)))

(defun wyjmij (osid &key co zczego)
  (setf (gdzie co) osid)
  (report osid t
          (lfn "Wyjmujesz ~a ~:[z~;ze~] ~a." (bier co) (syczace (dop zczego))
               (dop zczego))
          (lfn "~a wyjmuje ~a ~:[z~;ze~] ~a." (mian osid :poczatek-zdania t)
               (bier co) (syczace (dop zczego)) (dop zczego))))

(defun give-ok (osid &key co komu)
  (cond
    ((not (zob :zywy komu))
     (format nil "~a nie yje." (mian komu :poczatek-zdania t)))
    ((eql (gdzie komu :stop co) co)
     (if (eql (find-type co) :przedmioty)
         (format nil "~a ~[s~;jest~] w rodku ~a."
                 (mian komu :poczatek-zdania t)
                 (ecase (rodzaj komu)
                   (:nmos 0)
                   ((:m :z :n) 1))
                 (dop co))
         (format nil "~a niesie ~a." (mian co :poczatek-zdania t)
                 (bier komu))))
    ((nie-podniesie osid co))
    (t nil)))

(defun give (osid &key co komu)
  (setf (gdzie co) komu)
  (report osid t
          (lfn "Dajesz ~a ~a." (cel komu) (bier co))
          (lfn "~a daje ~a ~a." (mian osid :poczatek-zdania t)
               (cel komu) (bier co))))

(defun wez-ok (osid &key co)
  (cond
    ((eql (gdzie co :stop :os) osid)
     (format nil "Przecie ju trzymasz ~a." (bier co :bez-przymiotnikow t)))
    ((eql (gdzie osid :stop co) co)
     (if (eql (find-type co) :przedmioty)
         (format nil "Jeste w rodku ~a." (dop co))
         (format nil "~a ci niesie." (mian co :poczatek-zdania t))))
    ((nie-podniesie osid co))
    ((> (+ (- (ile-wazy osid) (zob :ciezar osid))
           (ile-wazy co))
        (zob :sila osid))
     "Niesiesz za duo.")
    (t nil)))

(defun wez (osid &key co)
  (setf (gdzie co) osid)
  (let ((v (ecase (find-type co)
             (:przedmioty 0)
             ((:postacie :gracz) 1))))
    (report osid t
            (lfn "~[Bierzesz~;Podnosisz~] ~a." v (bier co))
            (lfn "~a ~[bierze~;podnosi~] ~a."
                 (mian osid :poczatek-zdania t) v (bier co)))))

(defun wyrzuc-ok (osid &key co)
  (cond
    ((not (eql (gdzie co :stop :os) osid))
     (format nil "Nie trzymasz ~a." (dop co)))
    (t nil)))

(defun wyrzuc (osid &key co)
  (setf (gdzie co) (gdzie osid :stop :prz))
   (let ((v (ecase (find-type co)
             (:przedmioty 0)
             ((:postacie :gracz) 1))))
    (report osid t
            (lfn "~[Wyrzucasz~;Puszczasz~] ~a." v (bier co))
            (lfn "~a ~[wyrzuca~;puszcza~] ~a."
                 (mian osid :poczatek-zdania t) v (bier co)))))

(defun wejdz-do-ok (osid &key doczego)
  (cond
    ((not (zob :z_wnetrzem doczego))
     (format nil "Nie da si wej do ~a." (dop doczego)))
    ((eql doczego (zob :gdzie_prz osid))
     (format nil "Ju jeste w ~a." (miejsc doczego)))
    ((and (zob :otwieralne doczego) (not (zob :otwarte doczego)))
     (gk doczego "~a ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~]."))
    ((eql (gdzie doczego :stop osid) osid)
     (format nil "Nie moesz wej do czego co niesiesz."))
    (t nil)))

(defun wejdz-do (osid &key doczego)
  (setf (gdzie osid) doczego)
  (append
   (report osid nil
           (lfn "Wchodzisz do ~a." (dop doczego))
           (lfn "~a wchodzi do ~a."
                (mian osid :poczatek-zdania t) (dop doczego)))
   (let ((boat (get-label-id "boat"))
         (eb (get-label-id "eastbank"))
         (wb (get-label-id "westbank")))
     (when (and (eql osid *id-gracz*)
                (eql doczego boat)
                (find (gdzie boat) (list eb wb)))
       (setf (gdzie boat) (car (remove (gdzie boat) (list eb wb))))
       (list* "d przepywa rzek i zatrzymuje si na drugim brzegu."
              (desc-location (gdzie boat)))))))

(defun wyjdz-z-ok (osid &key zczego)
  (cond
    ((not (eql zczego (gdzie osid :stop zczego)))
     (format nil "Nie jeste w ~a." (miejsc zczego)))
    (t nil)))

(defun wyjdz-z (osid &key zczego)
  (setf (gdzie osid) (gdzie zczego :stop :prz))
  (report osid nil
          (lfn "Wychodzisz z ~a." (dop zczego))
          (lfn "~a wychodzi z ~a."
               (mian osid :poczatek-zdania t) (dop zczego))))

(defun nie-zyje-info (os)
  (if (eql os *id-gracz*)
      "Nie yjesz."
      (format nil "~a nie yje."
              (mian os :poczatek-zdania t :bez-przymiotnikow t))))

(defun nie-zyje (os)
  (when (not (zob :zywy os))
    (nie-zyje-info os)))

(defun powiedz-ok (osid &key dokogo cytat)
  (declare (ignore cytat osid))
  (cond
    ((nie-zyje dokogo))
    (t nil)))

(defun przyjaciel (os)
  (or (eql os *id-gracz*)
      (zob :przyjaciel os)))

(defun powiedz (osid &key dokogo cytat)
  (let ((fmt (if (eql dokogo *id-gracz*)
                 "~a~* mwi: \"~a\"."
                 "~a rozmawia z ~a."))
        (odpowiedz '()))
    (when (not (eql dokogo *id-gracz*))
      (if (cond
            ((eql dokogo (get-label-id "gollum")))
            ((and (przyjaciel osid) (przyjaciel dokogo))
             (> (random 10) 1))
            (t (eql 0 (random 10))))
          (push (cons osid cytat)
                (getf (gethash dokogo *npc-data*) :powiedziane))
          (when (> (random 6) 1)
            (setf odpowiedz (lfn "~a mwi: \"Nie\"."
                                 (mian dokogo :poczatek-zdania t
                                       :bez-przymiotnikow t))))))
    (report osid t
            (append (lfn "Rozmawiasz ~:[z~;ze~] ~a."
                         (syczace (narz dokogo)) (narz dokogo))
                    odpowiedz)
            (append (lfn fmt (mian osid :poczatek-zdania t)
                         (narz dokogo) cytat)
                    odpowiedz))))

(defun zabicie (os)
  (setf (zob :zywy os) nil)
  (setf (npc-data os :przymiotniki-za-zycia) (przymiotniki-opisujace os))
  (pushnew (get-label-id "martwy") (przymiotniki-opisujace os))
  (let ((g (gdzie os)))
    (dolist (id (append (przedmioty-trzyma os) (postacie-trzyma os)))
      (setf (gdzie id) g))))

(defun attackwith-ok (osid &key kogo czym)
  (cond
    ((nie-zyje kogo))
    ((and czym (nie-podniesie osid czym)))
    ((and (przyjaciel osid) (przyjaciel kogo) (not (eql osid *id-gracz*))
          (not (find kogo (npc-data osid :wrog)))))
    (t nil)))

(defparameter *starty-goblinow*
  '(("meangoblin" . #("dsp1" "dsp2"))
    ("nastygoblin" . #("dsp14"))
    ("hideousgoblin" . #("dsp2"))
    ("viciousgoblin" . #("dsp3" "dsp4" "dsp7"))
    ("horriblegoblin" . #("dsp3"))
    ("disgustinggoblin" . #("dsp6"))))

(defun start-goblina (goblin)
  (loop for (label . starty) in *starty-goblinow*
        when (eql goblin (get-label-id label))
        return (get-label-id (elt starty (random (length starty))))))

(defun zmartwychwstanie-goblina (goblin)
  (let ((oldlok (gdzie goblin))
        (newlok (start-goblina goblin))
        (retv '()))
    (when (eql (gdzie *id-gracz*) oldlok)
      (push "Goblin wpada do dziury i znika." retv))
    (setf (gdzie goblin) newlok)
    (setf (zob :zywy goblin) t)
    (setf (przymiotniki-opisujace goblin)
          (npc-data goblin :przymiotniki-za-zycia))
    (when (eql (gdzie *id-gracz*) newlok)
      (push (if (eql oldlok newlok)
                "Wchodzi inny goblin."
                (format nil "Wchodzi ~a." (mian goblin)))
            retv))
    (nreverse retv)))

(defun wynik-ataku (osid kogo czym sa so)
  (let ((opis (report osid t
                      (lfn "Atakujesz ~a~@[ ~a~]."
                           (bier kogo) (if czym (narz czym)))
                      (lfn "~a ~:[atakuje ~a~;~a atakuje~]~@[ ~a~]."
                           (mian osid :poczatek-zdania t)
                           (eql kogo *id-gracz*)
                           (bier kogo) (if czym (narz czym)))))
        (zabicie nil)
        (rk (ecase (rodzaj kogo)
              ((:m :n) 0)
              (:z 1))))
    (setf opis
          (append
           opis
           (cond
             ((or (> (- sa so) 100)
                  (and (> sa so) (eql 0 (random 5))))
              (let ((txt "Jednym starannie wymierzonym ciosem"))
                (setf zabicie t)
                (report
                 osid t
                 (lfn "~a rozupujesz ~[mu~;jej~] czaszk." txt rk)
                 (lfn "~a ~a rozupuje ~[mu~;jej~;ci~] czaszk."
                      txt (mian osid) (if (eql kogo *id-gracz*) 2 rk)))))
             ((or (> (- so sa) 100)
                  (and (> so (- sa 40)) (eql 0 (random 5))))
              (let ((txt "Lecz to daremny wysiek."))
                (report
                 osid t
                 (lfn "~a ~a broni si bez trudu." txt
                      (mian kogo :poczatek-zdania t))
                 (lfn "~a ~:[~a broni~;Bronisz~] si bez trudu." txt
                      (eql kogo *id-gracz*)
                      (mian kogo :poczatek-zdania t)))))
             (t
              (ecase (random 6)
                (0
                 (report
                  osid t
                  (lfn "Szybki cios wybija dech z ~a." (dop kogo))
                  (lfn "Szybki cios wybija ~:[dech z ~a~;z ciebie dech~]."
                       (eql kogo *id-gracz*) (dop kogo))))
                (1
                 (report
                  osid t
                  (lfn "Popychasz ~a. ~a potyka si, lecz szybko odzyskuje rwnowag."
                       (bier kogo) (mian kogo :poczatek-zdania t))
                  (lfn "~a ~:[popycha ~a~;~a popycha~].~@[ ~a~] ~:[potyka si~;Potykasz si~], lecz szybko ~:[odzyskuje~;odzyskujesz~] rwnowag."
                       (mian osid :poczatek-zdania t)
                       (eql kogo *id-gracz*) (bier kogo)
                       (unless (eql kogo *id-gracz*)
                         (mian kogo :poczatek-zdania t))
                       (eql kogo *id-gracz*) (eql kogo *id-gracz*))))
                (2
                 (let ((txt "jednak bez widocznego skutku"))
                   (report
                    osid t
                    (lfn "Zadajesz ~a cios, ~a." (cel kogo) txt)
                    (lfn "~a zadaje ~a cios, ~a."
                         (mian osid :poczatek-zdania t) (cel kogo) txt))))
                (3
                 (report
                  osid t
                  (lfn "Twoja pi prawie trafia ~a w ucho." (bier kogo))
                  (lfn "Pi ~a prawie trafia ~a w ucho."
                       (dop osid) (bier kogo))))
                (4
                 (report
                  osid t
                  (lfn "Zadajesz ~a paskudny cios w nog." (cel kogo))
                  (lfn "~a zadaje ~a paskudny cios w nog."
                       (mian osid :poczatek-zdania t) (cel kogo))))
                (5
                 (report
                  osid t
                  (lfn "Z du si trafiasz ~a w rami. ~a zatacza si i prawie pada."
                       (bier kogo) (mian kogo :poczatek-zdania t))
                  (lfn "~a z du si trafia ~a w rami. ~:[~a zatacza si i prawie pada.~;Zataczasz si i prawie padasz.~]"
                       (mian osid :poczatek-zdania t) (bier kogo)
                       (eql *id-gracz* kogo)
                       (mian kogo :poczatek-zdania t)))))))))
    (when (and zabicie opis)
      (setf opis (append opis (list (nie-zyje-info kogo)))))
    (values zabicie opis)))

(defun attackwith (osid &key kogo czym)
  (let ((sa (cond
              ((eql osid (kto-pierscien)) -100)
              (t (+ (or (zob :spr_boj osid) 0) 
                    (if czym (or (zob :przyd_boj czym) 0) 0)))))
        (so (or (zob :spr_boj kogo) 0))
        zabicie
        opis)
    (setf (values zabicie opis) (wynik-ataku osid kogo czym sa so))
    (when zabicie
      (zabicie kogo)
      (when (zob :goblin kogo)
        (setf opis (append opis (zmartwychwstanie-goblina kogo)))))
    (unless (or zabicie (eql kogo *id-gracz*))
      (pushnew osid (npc-data kogo :wrog)))
    opis))

(defun attack-ok (osid &key kogo)
  (attackwith-ok osid :kogo kogo))

(defun attack (osid &key kogo)
  (attackwith osid :kogo kogo))

(defun capture-ok (osid &key kogo)
  (or (attack-ok osid :kogo kogo)
      (and (zob :goblin osid)
           (eql (gdzie kogo) (get-label-id "gdungeon")))
      (and (zob :elf osid)
                 (eql (gdzie kogo) (get-label-id "edungeon")))))

(defun capture-dungeon (label osid kogo)
  (let* ((lok (get-label-id label))
         (retv (report
                osid t
                (lfn "Bierzesz ~a do niewoli." (bier kogo)) ; niemozliwe
                (lfn "~a bierze ~a do niewoli."
                     (mian osid :poczatek-zdania t) (bier kogo))
                (list lok)
                (ciemno-lfn "Pojawia si ~a." (mian kogo)))))
    (setf (gdzie kogo) lok)
    retv))

(defun capture (osid &key kogo)
  (cond
    ((zob :goblin osid) (capture-dungeon "gdungeon" osid kogo))
    ((zob :elf osid) (capture-dungeon "edungeon" osid kogo))
    (t (attack osid :kogo kogo))))

(defun przerzuc-ok (osid &key co przezco)
  (cond
    ((and (not (zob :otwarte przezco))
          (not (zob :rozbite przezco))
          (not (zob :rzeka przezco)))
     (gk przezco "~a ~[s~;jest~] ~[zamknite~;zamknity~;zamknita~]."))
    ((nie-podniesie osid co))
    ((nie-dosiegnie osid przezco))
    ((eql przezco (get-label-id "fastriver"))
     (format nil "Nie moesz przerzuci ~a przez ~a."
             (dop co :bez-przymiotnikow t) (bier przezco)))
    (t nil)))

(defun przerzuc (osid &key co przezco)
  (let ((dokad nil)
        (opis '())
        retv
        (mv nil))
    (cond
      ((and (eql przezco (get-label-id "fbriver"))
            (eql co (get-label-id "rope"))
            (progn
              (cond 
                ((eql (gdzie osid) (get-label-id "westbank"))
                 (setf dokad (get-label-id "eastbank")))
                ((eql (gdzie osid) (get-label-id "eastbank"))
                 (setf dokad (get-label-id "westbank"))))
              (when dokad
                (eql (gdzie (get-label-id "boat")) dokad))))
       (ecase (random 3)
         (0
          (setf (zob :zaczepione co) (get-label-id "boat"))
          (setf opis
                '("Lina przelatuje przez rzek i jej koniec wpada do odzi.")))
         (1
          (setf opis
                '("Lina przelatuje przez rzek i jej koniec wpada do odzi."
                  "Ale wyslizguje si do wody."))
          (setf (zob :zaczepione co) nil))
         (2
          (setf opis '("Lina prawie dolatuje do drugiego brzegu."))
          (setf (zob :zaczepione co) nil))))
      (t
       (setf dokad (car (remove (gdzie osid) (lokacje-z-drzwiami przezco))))
       (setf mv t)))
    (setf retv
          (append
           (report osid t
                   (lfn "Przerzucasz ~a przez ~a." (bier co) (bier przezco))
                   (lfn "~a przerzuca ~a przez ~a."
                        (mian osid :poczatek-zdania t)
                        (bier co) (bier przezco))
                   (list dokad)
                   (lfn "Kto wrzuca ~a." (bier co)))
           (when (eql (gdzie osid) (gdzie *id-gracz*))
             opis)))
    (when mv
      (setf (gdzie co) dokad))
    retv))

(defun ciagnij-ok (osid &key co)
  (declare (ignore osid))
  (cond
    ((not (eql co (get-label-id "rope")))
     "Nie moesz tego zrobi.")
    (t nil)))

(defun ciagnij (osid &key co)
  (let ((opis '()))
    (when (eql (zob :zaczepione co) (get-label-id "boat"))
      (setf opis '("d przepywa rzek i zatrzymuje si po tej stronie."))
      (setf (gdzie (get-label-id "boat")) (gdzie osid))
      (setf (zob :zaczepione co) nil))
    (append
     (report osid t
             (lfn "Cigniesz ~a." (bier co))
             (lfn "~a cignie ~a."
                  (mian osid :poczatek-zdania t) (bier co)))
     (when (eql (gdzie osid) (gdzie *id-gracz*))
       opis))))

(defun kto-pierscien ()
  (getf *history* :kto-pierscien))

(defun ust-kto-pierscien (os)
  (if (null os)
      (progn
        (setf (getf *history* :kto-pierscien) nil)
        (setf (getf *history* :pierscien-licznik) nil))
      (progn
        (setf (gdzie (get-label-id "ring")) os)
        (setf (getf *history* :kto-pierscien) os)
        (setf (getf *history* :pierscien-licznik) 0)))
  os)

(defsetf kto-pierscien ust-kto-pierscien)

(defun zaloz-ok (osid &key co)
  (cond
    ((not (eql co (get-label-id "ring")))
     "Nie moesz tego zrobi.")
    ((eql (kto-pierscien) osid)
     (format nil "Masz ju ~a na palcu." (bier co :bez-przymiotnikow t)))
    (t nil)))

(defun zaloz (osid &key co)
  (let ((retv (report osid t
                      (lfn "Zakadasz ~a." (bier co))
                      (lfn "~a zakada ~a."
                           (mian osid :poczatek-zdania t) (bier co)))))
    (setf (kto-pierscien) osid)
    retv))

(defun zdejm-ok (osid &key co)
  (cond
    ((not (eql co (get-label-id "ring")))
     "Nie moesz tego zrobi.")
    ((not (eql (kto-pierscien) osid))
     (format nil "Nie masz ~a na palcu" (dop co :bez-przymiotnikow t)))
    (t nil)))

(defun zdejm (osid &key co)
  (setf (kto-pierscien) nil)
  (report osid nil
          (lfn "Zdejmujesz ~a." (bier co))))

(defun breakdoorwith-ok (osid &key drzwi czym)
  (cond
    ((not (zob :rozbijalne drzwi))
     "Nie moesz tego zrobi.")
    ((zob :rozbite drzwi)
     (format nil "~a ~:[jest~;s~] ju ~a."
             (mian drzwi :bez-przymiotnikow t :poczatek-zdania t)
             (eql (rodzaj drzwi) :nmos)
             (zob :mian (zob :rozbite_przym drzwi) (rodzaj drzwi))))
    ((and czym (not (zob :lamiace czym)))
     (format nil "Nie moesz uderzy ~a w ~a."
             (narz czym) (bier drzwi)))
    ((and czym (nie-podniesie osid czym)))
    ((nie-dosiegnie osid drzwi))
    (t nil)))

(defun polamanie (co)
  (setf (zob :zlamane co) t)
  (when (zob :otwieralne co)
    (setf (zob :otwarte  co) t)
    (setf (zob :locked co) nil))
  (setf (zob :twardosc co) 0)
  (setf (zob :przyd_boj co) 0)
  (setf (przymiotniki-opisujace co)
        (list (zob :zlamane_przym co))))

(defun breakdoorwith (osid &key drzwi czym)
  (let ((opis 
         (report osid t
                 (lfn "Uderzasz ~@[~a ~]w ~a."
                      (if czym (narz czym))
                      (bier drzwi))
                 (lfn "~a uderza ~@[~a ~]w ~a."
                      (mian osid :poczatek-zdania t)
                      (if czym (narz czym))
                      (bier drzwi))
                 (lokacje-z-drzwiami drzwi)
                 (lfn "Kto uderza w ~a." (bier drzwi))))
        (twardosc (or (zob :twardosc drzwi) 0)))
    (when czym
      (setf twardosc (- twardosc (zob :twardosc czym)))
      (when (< twardosc 0)
        (setf twardosc 0)))
    (when (eql 0 (random (1+ twardosc)))
      (setf (zob :rozbite drzwi) t)
      (setf (przymiotniki-opisujace drzwi)
            (list (zob :rozbite_przym drzwi)))
      (when opis
        (setf opis (append opis (list (rozbite-info drzwi)))))
      (when (eql drzwi (get-label-id "web"))
        (when opis
          (setf opis (append opis (lfn "Pajki zaczynaj naprawia ~a."
                                       (bier drzwi)))))
        (setf (getf *history* :killing-web) nil)
        (setf (getf *history* :broken-web) 
              (+ 2 (random 3)))))
    (when (and czym
               (zob :lamalne czym)
               (not (zob :zlamane czym))
               (> 5 (random (1+ (zob :twardosc czym)))))
      (polamanie czym)
      (when opis
        (setf opis (append opis (list (zlamane-info czym))))))
    opis))

(defun breakdoor-ok (osid &key drzwi)
  (breakdoorwith-ok osid :drzwi drzwi))

(defun breakdoor (osid &key drzwi)
  (breakdoorwith osid :drzwi drzwi))

(defun polamczym-ok (osid &key co czym)
  (cond
    ((not (zob :lamalne co))
     "Nie moesz tego zrobi.")
    ((eql co czym)
     "Nie moesz tego zrobi.")
    ((zob :zlamane co)
     (format nil "~a ~:[jest~;s~] ju ~a."
             (mian co :bez-przymiotnikow t :poczatek-zdania t)
             (eql (rodzaj co) :nmos)
             (zob :mian (zob :zlamane_przym co) (rodzaj co))))
    ((and czym (not (zob :lamiace czym)))
     (format nil "Nie moesz uderzy ~a w ~a."
             (narz czym) (bier co)))
    ((and czym (nie-podniesie osid czym)))
    (t nil)))

(defun polamczym (osid &key co czym)
  (let ((opis
         (report osid t
                 (lfn "Uderzasz ~@[~a ~]w ~a."
                      (if czym (narz czym))
                      (bier co))
                 (lfn "~a uderza ~@[~a ~]w ~a."
                      (mian osid :poczatek-zdania t)
                      (if czym (narz czym))
                      (bier co))))
        (twardosc (or (zob :twardosc co) 0)))
    (when czym
      (setf twardosc (- twardosc (zob :twardosc czym))))
    (when (< twardosc 0)
      (setf twardosc 0))
    (when (eql 0 (random (1+ twardosc)))
      (polamanie co)
      (when opis
        (setf opis (append opis (list (zlamane-info co))))))
    (when (and czym
               (zob :lamalne czym)
               (not (zob :zlamane czym))
               (> 5 (random (1+ (zob :twardosc czym)))))
      (polamanie czym)
      (when opis
        (setf opis (append opis (list (zlamane-info czym))))))
    opis))

(defun polam-ok (osid &key co)
  (polamczym-ok osid :co co))

(defun polam (osid &key co)
  (polamczym osid :co co))

(defun shoot-ok (osid &key kogo bow arrow)
  (declare (ignore osid))
  (cond
    ((not (eql bow (get-label-id "bow")))
     (format nil "Nie moesz strzeli z ~a." (dop bow)))
    ((not (eql arrow (get-label-id "arrow")))
     (format nil "Nie moesz strzeli z ~a ~a." (dop bow) (narz arrow)))
    ((nie-zyje kogo))
    (t nil)))

(defun shoot (osid &key kogo bow arrow)
  (declare (ignore bow))
  (let ((n (if (eql osid (get-label-id "bard")) 1 3))
        rep
        opis)
    (setf rep (report osid t
                      (lfn "Strzelasz do ~a." (dop kogo))
                      (lfn "~a strzela do ~a."
                           (mian osid :poczatek-zdania t)
                           (dop kogo :dluga-forma t))))
    (setf (gdzie arrow) (gdzie osid))
    (if (eql 0 (random n))
        (progn
          (setf opis (list (format nil "~a ~:[trafia ~a~;~a trafia~]."
                                   (mian arrow :poczatek-zdania t
                                         :bez-przymiotnikow t)
                                   (eql kogo *id-gracz*)
                                   (bier kogo))
                           (nie-zyje-info kogo)))
          (zabicie kogo))
        (setf opis (lfn "~a nawet ~:[nie zahacza ~a~;~a nie zahacza~]."
                        (mian arrow :poczatek-zdania t
                              :bez-przymiotnikow t)
                        (eql kogo *id-gracz*)
                        (bier kogo))))
    (append
     rep
     (when (eql (gdzie osid) (gdzie *id-gracz*))
       opis))))

(defun burn-ok (osid &key kogo)
  (cond
    ((not (eql osid (get-label-id "dragon")))
     (format nil "Nie moesz spali ~a." (dop kogo)))
    ((nie-zyje kogo))
    (t nil)))

(defun burn (osid &key kogo)
  (let ((retv (append (report osid t
                              (lfn "Spalasz ~a." (bier kogo))
                              (lfn "~a ~:[spala ~a~;~a spala~]."
                                   (mian osid :poczatek-zdania t)
                                   (eql kogo *id-gracz*) (bier kogo)))
                      (list (nie-zyje-info kogo)))))
    (zabicie kogo)
    retv))

(defun drink-ok (osid &key co)
  (declare (ignore osid))
  (cond
    ((not (eql co (get-label-id "wine")))
     (format nil "Nie moesz pi ~a." (dop co)))
    (t nil)))

(defun drink (osid &key co)
  (setf (gdzie co) (get-label-id "nigdzie"))
  (report osid t
          (lfn "Wypijasz ~a." (bier co))
          (lfn "~a wypija ~a." (mian osid :poczatek-zdania t) (bier co))))

(defun eat-ok (osid &key co)
  (declare (ignore osid))
  (cond
    ((not (zob :food co))
     (format nil "Nie moesz zje ~a." (dop co)))
    (t nil)))

(defun eat (osid &key co)
  (let ((zjadl (or (npc-data osid :zjadl) 0))
        (gluttony nil)
        retv)
    (setf zjadl (+ zjadl 5 (random 6)))
    (setf (npc-data osid :zjadl) zjadl)
    (setf (gdzie co) (get-label-id "nigdzie"))
    (when (> zjadl 28)
      (setf gluttony t))
    (setf retv
          (report osid t
                  (append
                   (lfn "Zjadasz ~a." (bier co))
                   (when gluttony
                     (lfn "Obmierze akomstwo ci umiercio."))
                   (when gluttony
                     (list (nie-zyje-info osid))))
                  (append
                   (lfn "~a zjada ~a."
                        (mian osid :poczatek-zdania t) (bier co))
                   (when gluttony
                     (lfn "Obmierze akomstwo umiercio ~a."
                          (bier osid))))))
    (when gluttony
      (zabicie osid))
    retv))

(defun examine-ok (osid &key co)
  (declare (ignore osid co))
  nil)

(defun examine-magic-door (osid door)
  (cond
    ((eql (kto-pierscien) osid)
     (setf (getf *history* :magic-door) 4)
     (report osid nil
             (list "Ogldasz magiczne drzwi."
                   "Magiczne drzwi ostrzegaj, e nadchodz elfy.")))
    (t
     (report osid nil
             (append (lfn "Ogldasz ~a." (bier door))
                     (list "Nie widzisz niczego szczeglnego."))
             (append (lfn "~a oglda ~a."
                          (mian osid :poczatek-zdania t) (bier door))
                     (lfn "~a nie widzi niczego szczeglnego."
                          (mian osid :poczatek-zdania t)))))))

(defun read-map (osid map)
  (cond
    ((eql osid (get-label-id "elrond"))
     (let* ((i (getf *history* :map-missing))
            (mm (elt *map-missing* i)))
       (setf (zob (caddr mm) (get-label-id (car mm)))
             (get-label-id (cadr mm)))
       (report osid nil
               ()
               (append
                (lfn "~a oglda ~a."
                     (mian osid :poczatek-zdania t) (bier map))
                (lfn "~a mwi: \"~a\"."
                     (mian osid :poczatek-zdania t) (cadddr mm))))))
    (t
     (report osid nil
             (append
              (lfn "Ogldasz ~a." (bier map))
              '("S na niej jakie znaki, ale nie umiesz ich odczyta."))
             (append
              (lfn "~a oglda ~a."
                   (mian osid :poczatek-zdania t) (bier map))
              (lfn "S na niej jakie znaki, ale ~a nie umie ich odczyta."
                   (mian osid)))))))

(defun examine (osid &key co)
  (cond
    ((eql co (get-label-id "map"))
     (read-map osid co))
    ((eql co (get-label-id "magicdoor"))
     (examine-magic-door osid co))
    (t
     (report osid nil
             (append (lfn "Ogldasz ~a." (bier co))
                     (lfn "Widzisz ~a." (bier co)))
             (append (lfn "~a oglda ~a."
                          (mian osid :poczatek-zdania t) (bier co))
                     (lfn "~a widzi ~a."
                          (mian osid :poczatek-zdania t) (bier co)))))))

(defun dig-ok (osid &key piasek)
  (declare (ignore osid))
  (cond
    ((not (eql piasek (get-label-id "sand")))
     (format nil "Nie moesz tego zrobi." (dop piasek)))
    (t nil)))

(defun dig (osid &key piasek)
  (let ((zwn (zob :z_wnetrzem piasek)))
    (when (not zwn)
      (setf (zob :z_wnetrzem piasek) t
            (zob :otwarte piasek) t
            (gdzie (get-label-id "trapdoor")) piasek))
    (let ((rep (report osid t
                       (lfn "Rozkopujesz ~a." (bier piasek))
                       (lfn "~a rozkopuje ~a." 
                            (mian osid :poczatek-zdania t) (bier piasek)))))
      (append rep
              (if (and rep (not zwn))
                  (przedmiot-info piasek)
                  '())))))

