
;;  "Hobbit" wersja polska
;;  Copyright (C) 2005 Pawe Aleksander Fedoryski <pfedor@fuw.edu.pl>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(defvar *score*)

(defvar *uzyteat* (make-hash-table))
(defvar *npc-data*)
(defvar *history*)

(defun czy-polecenie (instr key)
  (setf instr (string-trim '(#\Space #\Newline #\Tab) instr))
  (equalp (get-const *polecenia* key) instr))

(defun clear-uzyteat (&optional os)
  (if os
      (setf (gethash os *uzyteat*) '())
      (setf *uzyteat* (make-hash-table))))

(defun plist-p (l &optional (forbidden-keys '()))
  (or (null l)
      (and (consp l)
           (keywordp (car l))
           (not (find (car l) forbidden-keys))
           (consp (cdr l))
           (plist-p (cddr l) (cons (car l) forbidden-keys)))))

(defun plists-equal (pl1 pl2)
  (let ((gs (gensym)))
    (or (equal pl1 pl2)
        (and (plist-p pl1)
             (plist-p pl2)
             (loop for (k v1) on pl1 by #'cddr with v2 do
                   (when (eql (setf v2 (getf pl2 k gs)) gs)
                     (return nil))
                   (unless (plists-equal v1 v2)
                     (return nil))
                   finally (return t))
             (loop for k in pl2 by #'cddr do
                   (when (eql (getf pl1 k gs) gs)
                     (return nil))
                   finally (return t))))))

(defun kasuj-uzyte (lw os)
  (set-difference lw (gethash os *uzyteat*) :test #'plists-equal))

(defun dodaj-do-uzytych (os elem)
  (push elem (gethash os *uzyteat*)))

(defun all-uniqueness (lw)
  (let* ((ms-all (remove-duplicates
                  (mapcar #'(lambda (elem)
                              (remove-if-not
                               #'(lambda (ms)
                                   (getf (getf elem ms) :all))
                               *ms-lista*))
                          lw)
                  :test #'equal)))
    (and (not (null lw))
         (= 1 (length ms-all))
         (= (length lw)
            (length
             (remove-duplicates lw
                                :key #'(lambda (elem)
                                         (mapcar
                                          #'(lambda (ms)
                                              (getf (getf elem ms) :obiekt))
                                          (car ms-all)))
                                :test #'equal)))
         (car lw))))

(defun elem-with-all (elem)
  (some #'(lambda (ms) (getf (getf elem ms) :all)) *ms-lista*))

(defun build-args (elem)
  (let ((dzial (find-object (getf elem :dzial))))
    (loop for (k v) on dzial by #'cddr
          when (search "->" (symbol-name k))
          append (list v (getf (getf elem (rm-arr k)) :obiekt)))))

(defun equiv-elems (el1 el2)
  (and (eql (getf el1 :szemrane) (getf el2 :szemrane))
       (equal (zob :funkcja (getf el1 :dzial))
              (zob :funkcja (getf el2 :dzial)))
       (plists-equal (build-args el1) (build-args el2))))

(defun wybierz-jednoznacznie (lw jestall)
  ; Ten when jest dodany na okolicznosc wersji bez polskich literek
  ; ("otworz skrzynie"). Z ogonkami to sie chyba nie moze zdarzyc.
  (when (and jestall
             (find-if-not #'elem-with-all lw))
    (setf lw (remove-if #'elem-with-all lw))
    (setf jestall nil))
  (setf lw (remove-duplicates lw :test #'equiv-elems))
  (sort lw #'> :key #'(lambda (elem) (getf elem :szemrane)))
  (loop
   (when (null lw)
     (return (values nil jestall)))
   (let ((ret (if jestall
                  (all-uniqueness lw)
                  (when (= 1 (length lw))
                    (car lw)))))
     (when ret
       (return (values ret jestall))))
   (setf lw (remove (getf (car lw) :szemrane) lw
                    :key #'(lambda (elem) (getf elem :szemrane))))))

(defun check-owner (elem os)
  (let ((kom nil))
    (dolist (ms *ms-lista*)
      (let* ((obiekt (getf (getf elem ms) :obiekt))
             (owner (and obiekt
                         (not (eql (find-type obiekt) :drzwi))
                         (gdzie obiekt :stop :osoba))))
        (when (and owner (not (find owner (list (gdzie os) os))))
          (setf kom (format nil "~a trzyma ~a."
                            (mian owner
                                  :bez-przymiotnikow t
                                  :poczatek-zdania t)
                            (bier obiekt :bez-przymiotnikow t)))
          (return))))
    kom))

(defun rm-arr (k)
  (let ((snk (symbol-name k)))
    (intern (subseq snk 0 (- (length snk) 2)) "KEYWORD")))

(defun czy-mozna (elem os)
  (let* ((dzial (find-object (getf elem :dzial)))
         (funkcja (intern (string-upcase (concatenate 'string
                                                      (getf dzial :funkcja)
                                                      "-ok"))))
         (args (build-args elem)))
    (apply funkcja os args)))

(defun sprawdz-ciemno (elem os)
  (when (and (eql os *id-gracz*)
             (czy-jest-ciemno)
             (not (zob :wciemnosci (getf elem :dzial))))
    "Nic nie widz. Jest ciemno."))

(defun wykonaj-dzialanie (os elem cytat)
  (let* ((dzial (find-object (getf elem :dzial)))
         (funkcja (intern (string-upcase (getf dzial :funkcja))))
         (args (build-args elem))
         (bylo-ciemno (czy-jest-ciemno))
         desc)
    (when cytat
      (setf args (append args (list :cytat cytat))))
    (setf desc (apply funkcja os args))
    (print-description
     (append
      (when (and (or (eql os *id-gracz*) (not bylo-ciemno))
                 (or (null desc)
                     (not (eql (car desc) :ciemno))))
        '(:ciemno))
      desc))))

(defun wykonaj-gramtab (os gramtab &optional (mowca nil))
  ;(format t "(wykonaj-gramtab ~s ~s ~s)~%" os gramtab mowca)
  (setf gramtab (zrob-zastapienia-gramtab gramtab))
  (let* ((acttab (interp os gramtab mowca))
         (lw (car acttab))
         (cytat (getf (cdr acttab) :cytat))
         (jestall (getf (cdr acttab) :jestall))
         wybrane)
    (when (null lw)
      (return-from wykonaj-gramtab (values "Nie rozumiem." nil)))
    (setf lw (kasuj-uzyte lw os))
    (labels ((smart-filter (test domyslny-kom)
               (let ((kom-razem '())
                     (kom-nsz '()))
                 (setf lw (remove-if #'(lambda (elem)
                                         (let ((kom (funcall test elem os)))
                                           (when kom
                                             (push kom kom-razem)
                                             (unless
                                                 (< 0 (getf elem :szemrane))
                                               (push kom kom-nsz))
                                             kom)))
                                     lw))
                 (when (null lw)
                   (return-from wykonaj-gramtab
                     (values
                      (cond ((= 1 (length kom-razem)) (car kom-razem))
                            ((= 1 (length kom-nsz)) (car kom-nsz))
                            (t domyslny-kom))
                      nil))))))
      (smart-filter #'sprawdz-ciemno "Nic nie widz. Jest ciemno.")
      (when (and (eql os *id-gracz*)
                 (not (eql (kto-pierscien) *id-gracz*)))
        (smart-filter #'check-owner "To niemoliwe."))
      (smart-filter #'czy-mozna "Nie moesz tego zrobi."))
    (unless (setf (values wybrane jestall)
                  (wybierz-jednoznacznie lw jestall))
      (return-from wykonaj-gramtab
        (values "Polecenie niejednoznaczne." nil)))
    (dodaj-do-uzytych os wybrane)
    (wykonaj-dzialanie os wybrane cytat)
    (values nil jestall)))

(defun dzialanie-postaci (os)
  (funcall (intern (string-upcase (zob :funkcja os))) os))

(defun move-world ()
  (dolist (id (type-ids :postacie))
    (when (zob :zywy id)
      (let ((grlok (gdzie *id-gracz*)))
        (dzialanie-postaci id)
        (when (and (not (eql grlok (gdzie *id-gracz*)))
                   (eql (find-type (gdzie *id-gracz*)) :mapa))
          (print-description (desc-location (gdzie *id-gracz*)))))))
  (zdarzenia))

(defun split-instr (instr)
  (let* ((len (length instr))
         (i 0)
         (buf (make-array len :fill-pointer 0 :element-type 'character))
         (retv '())
         (in-quotes nil))
    (loop (when (if (eql i len)
                    (notevery #'whitespacep buf)
                    (and (not in-quotes) (eql (elt instr i) #\.)))
            (push (copy-seq buf) retv)
            (setf (fill-pointer buf) 0))
          (when (eql i len)
            (return (nreverse retv)))
          (when (eql (elt instr i) #\")
            (setf in-quotes (not in-quotes)))
          (when (or in-quotes (not (eql (elt instr i) #\.)))
            (vector-push (elt instr i) buf))
          (incf i))))

(defun zastap (co czym s)
  (let ((lco (length co))
        (lczym (length czym))
        ls j (i 0))
    (loop (setf ls (length s))
          (loop (setf i (search co s :start2 i))
                (when (null i)
                  (return-from zastap s))
                (setf j (+ i lco))
                (when (and (or (eql i 0) (not (alpha-char-p (elt s (1- i)))))
                           (or (eql j ls) (not (alpha-char-p (elt s j)))))
                  (return))
                (incf i))
          (setf s (concatenate 'string (subseq s 0 i) czym (subseq s j)))
          (setf i (+ i lczym)))))

(defun zrob-zastapienia (s)
  (loop for (co . czym) in *zastapienia* do
        (setf s (zastap co czym s)))
  s)

(defun gramtabs-equal (gt1 gt2)
  (and (equal (getf (cdr gt1) :cytat)
              (getf (cdr gt2) :cytat))
       (eql (length (car gt1))
            (length (car gt2)))
       (every #'plists-equal (car gt1) (car gt2))))

(defun movement (osid s)
  (setf s (string-trim '(#\Space #\Newline #\Tab) s))
  (let ((k (or (assoc s *kierskr* :test #'equalp)
               (assoc (rozbior s) *kierzdania*
                      :test #'gramtabs-equal :key #'rozbior))))
    (when k
      (cond
        ((and (eql osid *id-gracz*)
              (czy-jest-ciemno))
         (let* ((lk '(:N :S :W :E :NE :NW :SE :SW :U :D))
                (k (nth (random (length lk)) lk)))
           (if (null (go-direction-ok osid :direction k))
               (print-description (go-direction osid :direction k))
               (if (= 0 (random 10))
                   (progn
                     (zabicie *id-gracz*)
                     (print-description
                      (list :ciemno
                            "Idziesz w losowym kierunku."
                            "Przewracasz si i rozbijasz sobie czaszk."
                            (nie-zyje-info *id-gracz*))))
                   (print-description
                    '(:ciemno
                      "Idziesz w losowym kierunku."
                      "Przewracasz si i uderzasz si w gow."
                      "Jest ciemno."))))))
        (t
         (let ((kom (go-direction-ok osid :direction (cdr k))))
           (when kom
             (error 'nie-rozumiem :opis kom)))
         (when (eql osid (get-label-id "bard"))
           (setf (npc-data osid :direction) (cdr k)))
         (print-description (go-direction osid :direction (cdr k)))))
      t)))

(defun zastap-elem-gramtab (elem)
  (cond
    ((and (equal (getf elem :rozkaz) (list (get-label-id "wziac")))
          (plists-equal
           (getf elem :docel)
           `(:rodz :z :rz ((:id ,(get-label-id "niewola") :rodz :z)))))
     (remf elem :docel)
     (setf (getf elem :rozkaz) (list (get-label-id "schwytac"))))
    ((and (find (getf elem :rozkaz) `((,(get-label-id "zalozyc"))
                                      (,(get-label-id "wlozyc")))
                :test #'equal)
          (plists-equal
           (getf elem :naco)
           `(:rodz :m :rz ((:id ,(get-label-id "palec") :rodz :m)))))
     (remf elem :naco)
     (setf (getf elem :rozkaz) (list (get-label-id "zalozyc"))))
    ((and (equal (getf elem :rozkaz) (list (get-label-id "zdjac")))
          (plists-equal
           (getf elem :zczego)
           `(:rodz :m :rz ((:id ,(get-label-id "palec") :rodz :m)))))
     (remf elem :zczego)))
  elem)

(defun zrob-zastapienia-gramtab (gramtab)
  (setf (car gramtab) (mapcar #'zastap-elem-gramtab (car gramtab)))
  gramtab)

(defun inventory ()
  (print-description
   (append '(:ciemno) (postac-info *id-gracz* :inventory t)))
  (let ((bylo-ciemno (czy-jest-ciemno)))
    (move-world)
    (ciemno-info bylo-ciemno)
    (print-description (ciemno-lfn ""))))

(defun print-score ()
  (format t "[SAMELINE]Przeszede ~,1F% tej przygody.~%~%"
          (* *score* 0.1)))

(defun a-moze-polecenie (s)
  (cond
    ((czy-polecenie s :inventory)
     (inventory)
     t)
    ((czy-polecenie s :score)
     (print-score)
     t)
    (t nil)))

(defun czy-jest-ciemno ()
  (and (zob :ciemno (gdzie *id-gracz*))
       (or (zob :zlamane (get-label-id "sword"))
           (not (eql (gdzie *id-gracz*)
                     (gdzie (get-label-id "sword")))))))

(defun ciemno-info (bylo-ciemno)
  (when (and (zob :zywy *id-gracz*) (czy-jest-ciemno))
    (print-description (if bylo-ciemno
                           (ciemno-lfn "Syszysz haas.")
                           (ciemno-lfn "Jest ciemno.")))))

(defun wykinstr (instr)
  (let ((sentences (split-instr instr))
        bylo-ciemno
        bylo-ci)
    (dolist (s sentences)
      (setf s (zrob-zastapienia s))
      (clear-uzyteat *id-gracz*)
      (setf bylo-ciemno (czy-jest-ciemno)
            bylo-ci nil)
      (cond
        ((a-moze-polecenie s)
         (when (not (zob :zywy *id-gracz*))
           (print-score)))
        ((movement *id-gracz* s)
         (when (not (zob :zywy *id-gracz*))
           (print-description (ciemno-lfn ""))
           (print-score)
           (return-from wykinstr))
         (when (and (not bylo-ciemno) (czy-jest-ciemno))
           (print-description (ciemno-lfn "Jest ciemno."))
           (setf bylo-ci t))
         (move-world)
         (unless bylo-ci
           (ciemno-info bylo-ciemno))
         (print-description (ciemno-lfn ""))
         (when (not (zob :zywy *id-gracz*))
           (print-score)))
        (t
         (let ((krzyczec t)
               (gramtab (rozbior s))
               powtorzyc
               kom)
           (loop
              (setf bylo-ciemno (czy-jest-ciemno)
                    bylo-ci nil)
              (setf (values kom powtorzyc)
                    (wykonaj-gramtab *id-gracz* gramtab))
              (when (not kom)
                (setf krzyczec nil)
                (when (and (not bylo-ciemno) (czy-jest-ciemno))
                  (print-description (ciemno-lfn "Jest ciemno."))
                  (setf bylo-ci t))
                (move-world)
                (unless bylo-ci
                  (ciemno-info bylo-ciemno))
                (print-description (ciemno-lfn ""))
                (when (not (zob :zywy *id-gracz*))
                  (print-score)
                  (return-from wykinstr)))
              (when (and kom krzyczec)
                (error 'nie-rozumiem :opis kom))
              (when (or kom (not powtorzyc))
                (return)))))))))

(defun prompt-read (prompt)
  (format *query-io* "[PROMPT]~a~%" prompt)
  (force-output *query-io*)
  (let ((retv (read-line *query-io* nil)))
    (when (null retv)
      (quit))
    retv))

(defun print-error (opis)
  (format t "[ERROR]~a~%" (string-upcase opis)))

(defun pause ()
  (format t "[PAUSE]~%"))

(defun clear ()
  (format t "[CLEAR]~%"))

(defun init-game ()
  (read-data *types*)
  (setf *score* 0)
  (setf *uzyteat* (make-hash-table))
  (setf *npc-data* (make-hash-table))
  (setf *history* '()))

(defun start-game ()
  (clear)
  (wykinstr "patrz"))

(defun game-loop (&optional no-init-game)
  (unless no-init-game
    (init-game))
  (start-game)
  (loop (handler-case
            (let ((instr (prompt-read "> ")))
              (when (czy-polecenie instr :quit)
                (return-from game-loop))
              (wykinstr instr))
          (nie-rozumiem (e) (print-error (opis e))))
     (when (or (not (zob :zywy *id-gracz*))
               (getf *history* :wygrales))
       (init-game)
       (pause)
       (start-game))))

