
#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <vector>
#include <gd.h>
using namespace std;

enum Color {
    BLACK = 128,
    BLUE = 129,
    RED = 130,
    MAGENTA = 131,
    GREEN = 132,
    CYAN = 133,
    YELLOW = 134,
    WHITE = 135
};

const int END_FRAME = 254;
const int END_ANIM = 255;
const int HLINE = 200;

struct RGB {
    Color c;
    int r;
    int g;
    int b;
} rgb[] = {
    { BLACK, 0, 0, 0 },
    { BLUE, 0, 0, 197 },
    { RED, 197, 0, 0 },
    { MAGENTA, 197, 0, 197 },
    { GREEN, 0, 194, 0 },
    { CYAN, 0, 194, 197 },
    { YELLOW, 197, 194, 0 },
    { WHITE, 197, 194, 197 }
};

Color get_pixel(gdImagePtr im, int j, int i)
{
    int c = gdImageGetPixel(im, j, i);
    int r = im->red[c];
    int g = im->green[c];
    int b = im->blue[c];
    for (int k = 0; k < int(sizeof(rgb)/sizeof(rgb[0])); ++k) {
        if (rgb[k].r == r  and  rgb[k].g == g  and  rgb[k].b == b)
            return rgb[k].c;
    }
    fprintf(stderr, "Unrecognized color: %d, %d, %d\n", r, g, b);
    exit(-1);
}

Color bgcolor;
vector<int> data;

void write_hline(int y, int x1, int x2)
{
    if (y == -1)
        return;
    if (x1 == x2) {
        data.push_back(y);
        data.push_back(x1);
    } else {
        data.push_back(HLINE);
        data.push_back(y);
        data.push_back(x1);
        data.push_back(x2);
    }
}

int main(int argc, char** argv)
{
    if (!argv[1]) {
        fprintf(stderr, "Usage: frompng DIR\n");
        exit(-1);
    }
    char* dir = argv[1];
    int framenum = 1;
    char pngname[100];
    Color prevframe[256][128];
    Color currframe[256][128];
    bool first = true;
    Color currcolor = BLACK;
    for (;;) {
        snprintf(pngname, sizeof(pngname), "%s/%08d.png", dir, framenum);
        FILE* in = fopen(pngname, "rb");
        if (!in) {
            if (errno == ENOENT)
                break;
            perror(pngname);
            exit(-1);
        }
        gdImagePtr im = gdImageCreateFromPng(in);
        if (!im) {
            fprintf(stderr, "Error\n");
            exit(-1);
        }
        fclose(in);
        if (first) {
            bgcolor = get_pixel(im, 0, 0);
            for (int i = 0; i < 128; ++i)
                for (int j = 0; j < 256; ++j)
                    prevframe[j][i] = bgcolor;
            data.push_back(bgcolor);
            first = false;
        }
        for (int i = 0; i < 128; ++i)
            for (int j = 0; j < 256; ++j)
                currframe[j][i] =  get_pixel(im, j, i);
        gdImageDestroy(im);
        
        int hline_y = -1, hline_x1, hline_x2;
        for (int i = 0; i < 128; ++i) {
            for (int j = 0; j < 256; ++j) {
                if (currframe[j][i] == prevframe[j][i])
                    continue;
                if (currframe[j][i] != currcolor) {
                    write_hline(hline_y, hline_x1, hline_x2);
                    hline_y = -1;
                    currcolor = currframe[j][i];
                    data.push_back(currcolor);
                }
                if (hline_y == -1) {
                    hline_y = i;
                    hline_x1 = j;
                    hline_x2 = j;
                } else if (i == hline_y  &&  j == hline_x2 + 1) {
                    hline_x2 = j;
                } else {
                    write_hline(hline_y, hline_x1, hline_x2);
                    hline_y = i;
                    hline_x1 = j;
                    hline_x2 = j;
                }
            }
        }
        write_hline(hline_y, hline_x1, hline_x2);
        data.push_back(END_FRAME);
        memcpy(prevframe, currframe, sizeof(prevframe));
        ++framenum;
    }
    data.push_back(END_ANIM);
    
    char outname[100];
    snprintf(outname, sizeof(outname), "%s-anim.c", dir);
    FILE* out = fopen(outname, "w");
    if (!out) {
        perror(outname);
        exit(-1);
    }
    fprintf(out, "\n\nextern unsigned char %s[];", dir);
    fprintf(out, "\nunsigned char %s[] = {\n    ", dir);
    bool firstbyte = true;
    int indent = 4;
    for (vector<int>::iterator i = data.begin(); i != data.end(); ++i) {
        if (!firstbyte)
            indent += fprintf(out, ", ");
        else
            firstbyte = false;
        if (indent > 65) {
            fprintf(out, "\n    ");
            indent = 4;
        }
        indent += fprintf(out, "%d", *i);
    }
    fprintf(out, "\n};\n");
    if (-1 == fclose(out)) {
        perror("fclose()");
        exit(-1);
    }
}
