
;;  "Hobbit" wersja polska
;;  Copyright (C) 2005 Pawe Aleksander Fedoryski <pfedor@fuw.edu.pl>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(define-condition nie-rozumiem (error)
  ((opis :initarg :opis :reader opis)))

(defun wez-cytat (s)
  (let ((ls (length s)))
    (do ((i (1+ (position-if-not #'whitespacep s))
            (+ i (if (eql (elt s i) #\") 2 1)))
         (res (make-array ls :fill-pointer 0 :element-type 'character)))
        ((if (eql i ls)
             (error 'nie-rozumiem :opis "Co nie tak z cudzysowami?")
             (and (eql (elt s i) #\") (or (eql (1+ i) ls)
                                          (not (eql (elt s (1+ i)) #\")))))
         (values res (subseq s (1+ i))))
      (vector-push (elt s i) res))))

(defun token-separator (c)
  (not (or (alpha-char-p c)
           (eql #\" c))))

(defun tokenize (s)
  (do (cytat w l)
      ((every #'token-separator s) (values (nreverse l) cytat))
    (if (eql (find-if-not #'token-separator s) #\")
        (if cytat
            (error 'nie-rozumiem :opis "Za duo cytatw.")
            (setf (values cytat s) (wez-cytat s)))
        (progn
          (setf (values w s)
                (get-one-word s :separator (complement #'alpha-char-p)))
          (push w l)))))

(defun a-moze-przyimek (sl seek)
  (dolist (ms (keys *ms-przim*))
    (when (equalp sl (get-const *ms-przim* ms))
      (unless (find ms seek)
        (push ms seek)
        (return-from a-moze-przyimek (values ms seek)))
      (error 'nie-rozumiem
             :opis (format nil "Za duo tych \"~a\"." sl))))
  (values nil seek))

(defun a-moze-oprocz (sl seekopr lw)
  (if (equalp sl *oprocz*)
      (progn
        (setf lw (delete :tmp lw
                         :key #'(lambda (elem) (getf elem :jakiopr))))
        (when (null lw)
          (error 'nie-rozumiem
                 :opis "To zdanie jest dla mnie zbyt skomplikowane."))
        (setf lw
              (mapcar
               #'(lambda (elem)
                   (setf (getf elem :jakiopr)
                         (cond
                           ((null (getf elem :jakiopr))
                            (or (getf elem :lastall) :tmp))
                           ((eql (getf elem :jakiopr) (getf elem :lastall))
                            :tmp)
                           (t
                            (getf elem :lastall))))
                   elem)
               lw))
        (values t t lw))
      (values nil seekopr lw)))

(defun sprawdz-slowo (sl)
  (let (spec)
    (loop for (k pl) on (list :mowca *mowca* :alln *alln*
                              :allnmos *allnmos* :allmos *allmos*)
          by #'cddr do
          (loop for (n s) on pl by #'cddr do
                (when (equalp sl s)
                  (push n (getf spec k)))))
    (append spec (dictionary sl))))

(defun rozpoznanie-slowa (sl seek seekopr spseek spseekopr)
  (let ((sprsl (sprawdz-slowo sl))
        rozk oprrz oprprz oprmowc rz prz all mowc)
    (labels ((obsluga-opr ()
               (when (find *oprprzyp* (getf sprsl :mowca))
                 (push t oprmowc))
               (dolist (v (getf sprsl :rzeczowniki))
                 (when (find *oprprzyp* (car v))
                   (push (cdr v) oprrz)))
               (dolist (v (getf sprsl :przymiotniki))
                 (when (find *oprprzyp* (car v))
                   (push (cdr v) oprprz))))
             (obsluga-ms (ms)
               (let ((p (get-const *ms-przyp* ms)))
                 (loop for (rr r) 
                       in '((:alln :n) (:allnmos :nmos) (:allmos :mos))
                       do (when (find p (getf sprsl rr))
                            (push r (getf all ms))))
                 (when (find p (getf sprsl :mowca))
                   (push t (getf mowc ms)))
                 (dolist (v (getf sprsl :rzeczowniki))
                   (when (find p (car v))
                     (push (cdr v) (getf rz ms))))
                 (dolist (v (getf sprsl :przymiotniki))
                   (when (find p (car v))
                     (push (cdr v) (getf prz ms)))))))
      (cond
        (spseek
         (obsluga-ms spseek)
         (unless (or rz prz all mowc)
           (error 'nie-rozumiem
                  :opis (format nil "Nie wiem co znaczy \"~a ~a\"."
                                (get-const *ms-przim* spseek) sl))))
        (spseekopr
         (obsluga-opr)
         (unless (or oprmowc oprrz oprprz)
           (error 'nie-rozumiem
                  :opis (format nil "Nie wiem co znaczy \"~a ~a\"."
                                *oprocz* sl))))
        (t
         (dolist (v (getf sprsl :czasowniki))
           (when (find :rozkaz (car v))
             (push (caddr v) rozk)))
         (when seekopr
           (obsluga-opr))
         (dolist (ms seek)
           (obsluga-ms ms))))
      (unless (or rozk oprrz oprprz oprmowc rz prz all mowc)
        (error 'nie-rozumiem
               :opis (format nil "Nie wiem co tu robi sowo \"~a\"." sl)))
      (values rozk oprrz oprprz oprmowc rz prz all mowc))))

(defun nanies-rozk (elem v)
  (push v (getf elem :rozkaz))
  elem)

(defun nanies-oprk (elem v k)
  (if (eql (getf elem :jakiopr) :tmp)
      (push v (getf (getf elem :oprtmp) k))
      (push v (getf (getf (getf elem (getf elem :jakiopr)) :opr) k)))
  elem)

(defun nanies-oprrz (elem v)
  (nanies-oprk elem v :rz))

(defun nanies-oprmowc (elem v)
  (nanies-oprk elem v :mowc))

(defun nanies-oprprz (elem v)
  (nanies-oprk elem v :prz))

(defun nanies-rz (elem v ms)
  (push v (getf (getf elem ms) :rz))
  (let ((id (getf v :id))
        (rodz (getf v :rodz)))
    (when (and (find rodz '(:mos :nmos))
               (null (getf (getf elem ms) :all)))
      (when (istnieje-lpoj id)
        (push rodz (getf (getf elem ms) :all))
        (setf (getf (getf elem ms) :niejawne) t))
      (setf (getf elem :lastall) ms)
      (when (eql :tmp (getf elem :jakiopr))
        (setf (getf elem :jakiopr) ms)
        (setf (getf (getf elem ms) :opr) (getf elem :oprtmp))
        (remf elem :oprtmp))))
  elem)

(defun nanies-prz (elem v ms)
  (push v (getf (getf elem ms) :prz))
  elem)

(defun nanies-all (elem v ms)
  (push v (getf (getf elem ms) :all))
  (setf (getf elem :lastall) ms)
  (when (eql :tmp (getf elem :jakiopr))
    (setf (getf elem :jakiopr) ms)
    (setf (getf (getf elem ms) :opr) (getf elem :oprtmp))
    (remf elem :oprtmp))
  elem)

(defun nanies-mowc (elem v ms)
  (push v (getf (getf elem ms) :mowc))
  elem)

(defun all-too-long (ems)
  (> (length (getf ems :all))
     (if (getf ems :niejawne) 2 1)))

(defun mowc-nie-samo (ems)
  (and (getf ems :mowc)
       (/= 0 (reduce #'+ (mapcar #'(lambda (k) (length (getf ems k)))
                                 '(:rz :prz :all))))))

(defun rodzaje-ems (ems)
  (let* ((rodz-all (getf ems :all))
         (rz (getf ems :rz))
         (prz (getf ems :prz))
         (rodz-rz (mapcar #'(lambda (v) (getf v :rodz)) rz))
         (rodz-prz (mapcar #'(lambda (v) (getf v :rodz)) prz))
         (rodz (append rodz-all rodz-rz rodz-prz)))
    (remove-duplicates rodz)))

(defun rozne-rodzaje (ems)
  (> (length (rodzaje-ems ems)) 1))

(defun rodzaje-opr (ems)
  (let* ((rz (getf (getf ems :opr) :rz))
         (prz (getf (getf ems :opr) :prz))
         (rodz-rz (mapcar #'(lambda (v) (getf v :rodz)) rz))
         (rodz-prz (mapcar #'(lambda (v) (getf v :rodz)) prz))
         (rodz (append rodz-rz rodz-prz)))
    (remove-duplicates rodz)))

(defun rozne-rodzaje-opr (ems)
  (> (length (rodzaje-opr ems)) 1))

(defun niedopasowane-oprocz (elem)
  (let ((jakiopr (getf elem :jakiopr)))
    (when jakiopr
      (or (eql jakiopr :tmp)
          (null (getf (getf elem jakiopr) :opr))))))

(defun invalid-elem (elem)
  (or (niedopasowane-oprocz elem)
      (/= (length (getf elem :rozkaz)) 1)
      (some #'(lambda (ms)
                (let ((ems (getf elem ms)))
                  (or (all-too-long ems)
                      (mowc-nie-samo ems)
                      (rozne-rodzaje ems)
                      (rozne-rodzaje-opr ems)
                      (> (length (getf ems :rz)) 1))))
            *ms-lista*)))

(defun czysc (lw)
  (remove-if #'invalid-elem lw))

(defun uzupelnij-elem-rodz (elem)
  (dolist (ms *ms-lista*)
    (let ((ems (getf elem ms)))
      (when ems
        (setf (getf (getf elem ms) :rodz) (car (rodzaje-ems ems)))
        (when (getf ems :opr)
          (setf (getf (getf (getf elem ms) :opr) :rodz)
                (car (rodzaje-opr ems)))))))
  elem)

(defun uzupelnij-elem-all (elem)
  (dolist (ms *ms-lista*)
     (let* ((ems (getf elem ms))
            (rodz (getf ems :rodz))
            (all (getf ems :all))
            (opr (getf ems :opr))
            (rz (getf ems :rz)))
       (when (and ems (null all))
         (when (or (and (find rodz '(:nmos :mos))
                        (null rz))
                   opr)
           (setf (getf (getf elem ms) :all) (list rodz))))))
  elem)

(defun uzupelnij-elem (elem)
  (setf elem (uzupelnij-elem-all (uzupelnij-elem-rodz elem)))
  (remf elem :lastall)
  (remf elem :jakiopr)
  elem)

(defun uzupelnij (lw)
  (mapcar #'uzupelnij-elem lw))

(defun expand (ll)
  (loop for (l f pl-p) on ll by #'cdddr append
        (if pl-p
            (loop for (ms lms) on l by #'cddr append
                  (list lms f (list ms)))
            (list l f '()))))

(defun rozbior (instrukcja)
  (let ((lw (list '()))
        slowa cytat
        (seek (remove-if #'(lambda (ms) (get-const *ms-przim* ms))
                         *ms-lista*))
        (seekopr nil)
        (spseek nil)
        (spseekopr nil))
    (setf (values slowa cytat) (tokenize instrukcja))
    (dolist (sl slowa)
      ;(format
      ; t "lw: ~s~%seek: ~s, seekopr: ~s, spseek: ~s, spseekopr: ~s~%sl: ~s~%"
      ; lw seek seekopr spseek spseekopr sl)
      (block slowo
        (unless (or spseek spseekopr)
          (when (setf (values spseek seek)
                      (a-moze-przyimek sl seek))
            (return-from slowo))
          (when (setf (values spseekopr seekopr lw)
                      (a-moze-oprocz sl seekopr lw))
            (return-from slowo)))
        (multiple-value-bind
              (rozk oprrz oprprz oprmowc rz prz all mowc)
            (rozpoznanie-slowa sl seek seekopr spseek spseekopr)
          (setf spseek nil spseekopr nil)
          (setf lw
                (loop for (l f rest)
                      on (expand (list rozk #'nanies-rozk nil
                                       oprrz #'nanies-oprrz nil
                                       oprprz #'nanies-oprprz nil
                                       oprmowc #'nanies-oprmowc nil
                                       rz #'nanies-rz t
                                       prz #'nanies-prz t
                                       all #'nanies-all t
                                       mowc #'nanies-mowc t))
                      by #'cdddr append
                      (mapcan #'(lambda (v)
                                  (mapcar #'(lambda (elem)
                                              (apply f (list*
                                                        (copy-tree elem) v
                                                        rest)))
                                          lw))
                              l))))))
    (setf lw (czysc lw))
    (when (null lw)
      (error 'nie-rozumiem :opis "Nie rozumiem"))
    (setf lw (uzupelnij lw))
    (list lw :cytat cytat)))


