
;;  "Hobbit" wersja polska
;;  Copyright (C) 2005 Pawe Aleksander Fedoryski <pfedor@fuw.edu.pl>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(defmacro npc-data (os k)
  `(getf (gethash ,os *npc-data*) ,k))

(defun odwet (os)
  (some #'(lambda (w)
            (and (zob :zywy w)
                 (eql (gdzie w) (gdzie os))
                 (not (eql w (kto-pierscien)))
                 (or (try 'burn os :kogo w)
                     (and (eql (gdzie (get-label-id "bow") :stop :os) os)
                          (eql (gdzie (get-label-id "arrow") :stop :os) os)
                          (try 'shoot os :kogo w :bow (get-label-id "bow")
                               :arrow (get-label-id "arrow")))
                     (try 'attack os :kogo w))))
        (npc-data os :wrog)))

(defun npc-wyk-gramtab (os)
  (if (null (npc-data os :gramtab))
      nil
      (destructuring-bind (mowca . gramtab) (npc-data os :gramtab)
        (if (null (wykonaj-gramtab os gramtab mowca)) ; OK: nie ma komunikatu
            t
            (progn
              (clear-uzyteat os)
              (setf (npc-data os :gramtab) nil)
              nil)))))

(defun npc-wyk-instr (os)
  (loop (when (null (npc-data os :instrukcja))
          (return nil))
        (destructuring-bind (mowca . s)
            (pop (npc-data os :instrukcja))
          (when (handler-case (movement os s)
                  (nie-rozumiem () t))
            (return t))
          (let ((gramtab (handler-case (rozbior (zrob-zastapienia s))
                           (nie-rozumiem () nil))))
            (when gramtab
              (setf (npc-data os :gramtab) (cons mowca gramtab))
              (when (npc-wyk-gramtab os)
                (return t)))))))
  
(defun npc-wyk-powiedziane (os)
  (loop (when (null (npc-data os :powiedziane))
          (return nil))
        (destructuring-bind (mowca . cytat)
            (pop (npc-data os :powiedziane))
          (let ((instr (split-instr cytat)))
            (when instr
              (setf (npc-data os :instrukcja)
                    (mapcar #'(lambda (s) (cons mowca s)) instr))
              (when (npc-wyk-instr os)
                (return t)))))))

(defun zalegle-polecenia (os)
  (or (npc-wyk-gramtab os)
      (npc-wyk-instr os)
      (npc-wyk-powiedziane os)))

(defun sa-zalegle-polecenia (os)
  (or (npc-data os :powiedziane)
      (npc-data os :instrukcja)
      (npc-data os :gramtab)))

(defun gw ()
  (not (eql (kto-pierscien) *id-gracz*)))

(defun try (fn os &rest args)
  (let ((fn-ok (intern (concatenate 'string (symbol-name fn) "-OK"))))
    (cond
      ((apply fn-ok os args) nil)
      (t
       (let ((bylo-ciemno (czy-jest-ciemno))
             (desc (apply fn os args)))
         (print-description
          (append
           (when (and (not bylo-ciemno)
                      (or (null desc)
                          (not (eql (car desc) :ciemno))))
             '(:ciemno))
           desc))
         t)))))

(defun random-movement (os &optional locs)
  (when (eql (find-type (gdzie os)) :mapa)
    (let ((exits (remove-if-not
                  #'(lambda (d)
                      (let ((dest (find-destination (gdzie os) d)))
                        (and dest
                             (or (null locs)
                                 (find dest locs :key #'get-label-id)))))
                  (keys *kierunki*))))
      ;(format t "~s ~s ~s~%" os locs exits)
      (when exits
        (let ((d (nth (random (length exits)) exits)))
          (try 'go-direction os :direction d))))))

(defun wyjscie-z-przedmiotu (os)
  (let ((prz (gdzie os :stop :przedmiot)))
    (when (and (eql (find-type prz) :przedmioty)
               (find prz (co-widzi os))
               (zob :otwarte prz))
      (try 'wyjdz-z os :zczego prz))))

(defun fgandalf (os)
  (let ((map (get-label-id "map"))
        (hall (get-label-id "hall"))
        (rgdoor (get-label-id "green"))
        (cwo (co-widzi os)))
    (when (and (= 0 (random 15))
               (npc-data os :take-map)
               (npc-data os :co-to-jest)
               (npc-data os :give-map-2))
      (setf (npc-data os :take-map) nil
            (npc-data os :co-to-jest) nil
            (npc-data os :give-map-2) nil))
    (or (odwet os)
        (zalegle-polecenia os)
        (wyjscie-z-przedmiotu os)
        (and (not (npc-data os :give-map-1))
             (find *id-gracz* cwo)
             (eql (gdzie map :stop :osoba) os)
             (setf (npc-data os :give-map-1)
                   (try 'give os :co map :komu *id-gracz*)))
        (and (not (npc-data os :open-door))
             (every #'(lambda (l) (eql (gdzie (get-label-id l)) hall))
                    '("gandalf" "thorin" "gracz"))
             (setf (npc-data os :open-door)
                   (try 'otwdrzwi os :drzwi rgdoor)))
        (and (not (npc-data os :go-east))
             (setf (npc-data os :go-east)
                   (try 'go-direction os :direction :E)))
        (and (not (npc-data os :take-map))
             (find map cwo)
             (setf (npc-data os :take-map)
                   (try 'wez os :co map)))
        (and (not (npc-data os :co-to-jest))
             (npc-data os :take-map)
             (setf (npc-data os :co-to-jest) t)
             (find *id-gracz* cwo)
             (try 'powiedz os :dokogo *id-gracz* :cytat "Co to jest?"))
        (and (not (npc-data os :close-door))
             (notany #'(lambda (l) (eql (gdzie (get-label-id l)) hall))
                     '("gandalf" "thorin" "gracz"))
             (find rgdoor cwo)
             (setf (npc-data os :close-door)
                   (try 'zamkdrz os :drzwi rgdoor)))
        (and (not (npc-data os :give-map-2))
             (eql (gdzie map :stop :osoba) os)
             (cond
               ((eql (gdzie os) (get-label-id "polanka"))
                (setf (npc-data os :give-map-2)
                      (try 'wyrzuc os :co map)))
               ((find *id-gracz* cwo)
                (setf (npc-data os :give-map-2)
                      (try 'give os :co map :komu *id-gracz*)))
               (t nil)))
        (and (find *id-gracz* cwo)
             (= 0 (random 5))
             (try 'powiedz os :dokogo *id-gracz*
                  :cytat (cond
                           ((= 0 (random 6))
                            "Robisz wspania robot")
                           ((= 0 (random 3))
                            "Pospiesz si")
                           (t "Witaj"))))
        (and (= 0 (random 7))
             (try 'wait os))
        (and (find (get-label-id "vwarg") cwo)
             (= 0 (random 7))
             (try 'attack os :kogo (get-label-id "vwarg")))
        (and (> (random 7) 0)
             (random-movement os)))))

(defun find-direction (lok1 lok2)
  (find-if #'(lambda (d) (eql (find-destination lok1 d) lok2))
           (keys *kierunki*)))

(defun fthorin (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (wyjscie-z-przedmiotu os)
      (let ((oslok (gdzie os))
            (grlok (gdzie *id-gracz*))
            (key (get-label-id "smallcuriouskey")))
        (or (and (find key (co-widzi os))
                 (not (eql (gdzie key :stop :osoba) os))
                 (try 'wez os :co key))
            (and (eql (gdzie key :stop :osoba) os)
                 (not (npc-data os :thrains-key))
                 (eql oslok grlok)
                 (gw)
                 (> (random 10) 0)
                 (setf (npc-data os :thrains-key) t)
                 (try 'powiedz os :dokogo *id-gracz* :cytat
                      "To by klucz Thraina"))
            (and (not (eql oslok grlok))
                 (eql :mapa (find-type oslok))
                 (eql :mapa (find-type grlok))
                 (gw)
                 (let ((d (find-direction oslok grlok)))
                   (and d 
                        (try 'go-direction os :direction d))))
            (and (not (gw))
                 (eql oslok grlok)
                 (or (and (not (npc-data os :gdzie-wlamywacz))
                          (> (random 10) 1))
                     (= 0 (random 7)))
                 (progn
                   (setf (npc-data os :gdzie-wlamywacz) t)
                   (try 'powiedz os :dokogo *id-gracz* :cytat
                        "Gdzie si podzia wamywacz?")))
            (and (eql oslok grlok)
                 (gw)
                 (= 0 (random 4))
                 (try 'powiedz os :dokogo *id-gracz* :cytat
                      "Pospiesz si"))
            (and (eql oslok grlok)
                 (= 0 (random 5))
                 (progn
                   (print-description
                    (lfn "~a siada i zaczyna piewa pie o zocie."
                         (mian os :poczatek-zdania t)))
                   t))
            (and (or (not (eql oslok grlok))
                     (not (gw)))
                 (not (eql oslok (get-label-id "ldcave")))
                 (= 0 (random 3))
                 (random-movement os))))
      (and (= 0 (random 3))
           (try 'wait os))))

(defun ftroll (os)
  (when (odwet os)
    (return-from ftroll))
  (when (zalegle-polecenia os)
    (return-from ftroll))
  (let ((polanka (get-label-id "polanka")))
    (when (and (eql (gdzie os) (gdzie *id-gracz*))
               (getf *history* :trolle-jesc)
               (zob :zywy *id-gracz*))
      (print-description
       (list (format nil "~a ci zjada." (mian os :poczatek-zdania t))
             (format nil "Obmierze akomstwo umiercio ~a." (bier  os))
             (nie-zyje-info *id-gracz*)))
      (zabicie *id-gracz*)
      (return-from ftroll))
    (when (and (eql (gdzie (get-label-id "htroll")) polanka)
               (eql (gdzie (get-label-id "vtroll")) polanka)
               (eql (gdzie *id-gracz*) polanka))
      (when (not (getf *history* :trolle-rozmowa))
        (setf (getf *history* :trolle-rozmowa) t)
        (try 'powiedz os :dokogo *id-gracz* :cytat
             "Rety, patrzaj! Nada si toto na piecze?")
        (return-from ftroll))
      (try 'powiedz os :dokogo *id-gracz* :cytat
           "Mona sprbowa, ale nie starczy bodaj raz na zb"))))

(defun felrond (os)
  (let ((cnt (or (npc-data os :counter) 0)))
    (when (or (not (npc-data os :hello)) (not (npc-data os :lunch)))
      (setf cnt 0))
    (incf cnt)
    (when (eql cnt 4)
      (setf cnt 0)
      (setf (npc-data os :lunch) nil)
      (setf (npc-data os :hello) nil))
    (setf (npc-data os :counter) cnt))
  (or (odwet os)
      (zalegle-polecenia os)
      (and (eql (gdzie os) (gdzie *id-gracz*))
           (not (npc-data os :hello))
           (setf (npc-data os :hello) t)
           (try 'powiedz os :dokogo *id-gracz* :cytat "Witaj"))
      (and (eql (gdzie (get-label-id "lunch")) (get-label-id "nigdzie"))
           (eql (gdzie os) (gdzie *id-gracz*))
           (gw)
           (not (npc-data os :lunch))
           (setf (npc-data os :lunch) t)
           (try 'give os :co (get-label-id "lunch") :komu *id-gracz*))))

(defun try-capture (os)
  (some #'(lambda (kogo) (try 'capture os :kogo kogo))
        (remove-if #'(lambda (o)
                       (or (and (not (eql o *id-gracz*))
                                (not (eql (find-type o) :postacie)))
                           (not (zob :zywy o))
                           (and (zob :elf o) (zob :elf os))
                           (and (zob :goblin o) (zob :goblin os))
                           (and (eql o (get-label-id "gollum"))
                                (zob :goblin os))))
                   (co-widzi os))))

(defun fgoblin (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (and (= (random 4) 0)
           (find *id-gracz* (co-widzi os))
           (try 'attack os :kogo *id-gracz*))
      (and (> (random 10) 0)
           (try-capture os))
      (and (> (random 4) 0)
           (random-movement os '("ldcave" "gdungeon" "dwp" "dllake"
                                 "dsp1" "dsp2" "dsp3" "dsp4" "dsp5"
                                 "dsp6" "dsp7" "dsp8" "dsp9" "dsp10"
                                 "dsp11" "dsp12" "dsp13" "dsp14" "dsp15"
                                 "dsp16" "iggate" "oggate")))
      (and (= (random 3) 0)
           (random-movement os))))

(defun nastygoblin-cycle (os)
  (let ((crack (get-label-id "crack"))
        (dsp14 (get-label-id "dsp14"))
        (dsp13 (get-label-id "dsp13"))
        (ldcave (get-label-id "ldcave")))
    (cond
      ((eql (gdzie os) dsp14)
       (setf (npc-data os :up) t)
       (try 'go-direction os :direction :N))
      ((and (eql (gdzie os) dsp13)
            (zob :otwarte crack))
       (if (npc-data os :up)
           (try 'go-direction os :direction :U)
           (try 'zamkdrz os :drzwi crack)))
      ((and (eql (gdzie os) dsp13)
            (not (zob :otwarte crack)))
       (if (npc-data os :up)
           (try 'otwdrzwi os :drzwi crack)
           (try 'go-direction os :direction :S)))
      ((eql (gdzie os) ldcave)
       (setf (npc-data os :up) nil)
       (try 'go-direction os :direction :D)))))
                 
(defun fnastygoblin (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (try-capture os)
      (nastygoblin-cycle os)))

(defun fwoodelf (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (and (> (random 10) 0)
           (try-capture os))
      (and (> (random 4) 0)
           (random-movement os '("fr1" "fr2" "fr3"
                                 "waterfall" "runningriver")))))

(defun fdragon (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (and (getf *history* :smok-zly)
           (eql (gdzie *id-gracz*) (gdzie os))
           (or (and (gw)
                    (= (random 7) 0)
                    (try 'burn os :kogo *id-gracz*))
               (and (not (getf *history* :smok-1))
                    (gw)
                    (try 'powiedz os :dokogo *id-gracz* :cytat
                         "No c, zodzieju, tym razem twoje knowania na nic. Szykuj si na mier.")
                    (setf (getf *history* :smok-1) t))
               (and (not (getf *history* :smok-2))
                    (not (gw))
                    (try 'powiedz os :dokogo *id-gracz* :cytat
                         "Nie mog ci zobaczy, zodzieju, ale wci mog ci spali. Szykuj si na mier.")
                    (setf (getf *history* :smok-2) t
                          (getf *history* :smok-1) t))
               (and (gw)
                    (> (random 7) 1)
                    (try 'burn os :kogo *id-gracz*))))
      (and (> (random 3) 0)
           (random-movement os '("dragonsdesolation"
                                 "lowerhalls"
                                 "dale"
                                 "ravenhill"
                                 "frontgate"
                                 "lonelymountain")))))

(defun butler-cycle (os)
  (when (not (eql (gdzie os) (get-label-id "cellar")))
    (return-from butler-cycle nil))
  (if (npc-data os :door)
      (let ((door (get-label-id "reddoor"))
            (key (get-label-id "redkey")))
        (or (and (eql (gdzie key) (gdzie os))
                 (try 'otwdrzkl os :drzwi door :klucz key))
            (try 'zamkdrz os :drzwi door)
            (and (eql (gdzie key) (gdzie os))
                 (try 'zamkdrzkl os :drzwi door :klucz key)
                 (progn
                   (setf (npc-data os :door) nil)
                   t))
            (setf (npc-data os :door) nil)))
      (let ((ltd (get-label-id "ltrapdoor"))
            (barrel (get-label-id "barrel"))
            (wine (get-label-id "wine")))
        (if (not (eql (gdzie barrel) (gdzie os)))
            (progn
              (when (zob :otwarte ltd)
                (try 'zamkdrz os :drzwi ltd))
              (setf (npc-data os :door) t))
            (or (and (not (zob :otwarte barrel))
                     (eql (gdzie wine) barrel)
                     (try 'otwprz os :przedmiot barrel))
                (and (zob :otwarte barrel)
                     (eql (gdzie wine :stop :przedmiot) barrel)
                     (try 'drink os :co wine))
                (and (zob :otwarte barrel)
                     (not (eql (gdzie wine :stop :przedmiot) barrel))
                     (try 'zamkprz os :przedmiot barrel))
                (and (not (zob :otwarte barrel))
                     (not (eql (gdzie wine) barrel))
                     (not (zob :otwarte ltd))
                     (try 'otwdrzwi os :drzwi ltd))
                (and (not (zob :otwarte barrel))
                     (not (eql (gdzie wine) barrel))
                     (zob :otwarte ltd)
                     (not (eql (gdzie barrel :stop os) os))
                     (try 'wez os :co barrel))
                (and (not (zob :otwarte barrel))
                     (not (eql (gdzie wine) barrel))
                     (zob :otwarte ltd)
                     (eql (gdzie barrel :stop os) os)
                     (try 'przerzuc os :co barrel :przezco ltd)))))))

(defun fbutler (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (try-capture os)
      (butler-cycle os)))

(defun fwarg (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (and (find *id-gracz* (co-widzi os))
           (= 0 (random 7))
           (try 'attack os :kogo *id-gracz*))
      (and (find *id-gracz* (co-widzi os))
           (= 0 (random 5))
           (progn
             (print-description
              (lfn "~a biega dookoa ciebie i wyje."
                   (mian os :poczatek-zdania :t)))
             t))
      (random-movement os)))

(defun fgollum (os)
  ;(format t "~a~%" (npc-data os :powiedziane))
  (when (npc-data os :oczekiwana-odpowiedz)
    (unless (and (not (null (npc-data os :powiedziane)))
                 (eql *id-gracz* (caar (npc-data os :powiedziane)))
                 (equalp (cdar (npc-data os :powiedziane))
                         (npc-data os :oczekiwana-odpowiedz)))
      (setf (getf *history* :gollum-zabic) t))
    (setf (npc-data os :oczekiwana-odpowiedz) nil)
    (return-from fgollum))
  (setf (npc-data os :powiedziane) nil)
  (or (odwet os)
      (when (not (npc-data os :start))
        (when (eql (gdzie *id-gracz*) (get-label-id "dsp10"))
          (try 'go-direction os :direction :SW)
          (setf (npc-data os :start) t))
        t)
      (when (and (eql (gdzie *id-gracz*) (gdzie os))
                 (not (npc-data os :mumbling))
                 (not (eql (gdzie (get-label-id "ring") :stop :os) os))
                 (< 0 (random 6)))
        (setf (npc-data os :mumbling) t)
        (try 'powiedz os :dokogo *id-gracz* :cytat
             (if (< 0 (random 10))
                 "Ccco on ma w kieszeni?"
                 "Twj urodzinowy dar! Jak to si ssstao, e go zgubie, mj ssskarbie?")))
      (when (and (find (get-label-id "ring") (co-widzi os))
                 (not (npc-data os :take-ring)))
        (setf (npc-data os :take-ring) t)
        (try 'wez os :co (get-label-id "ring")))
      (when (and (find *id-gracz* (co-widzi os))
                 (not (npc-data os :question)))
        (setf (npc-data os :question) t)
        (if (< 0 (random 8))
            (progn
              (setf (npc-data os :oczekiwana-odpowiedz) "czowiek")
              (try 'powiedz os :dokogo *id-gracz* :cytat
                   "Jakie zwierz ma cztery nogi o poranku, dwie w cigu dnia a trzy wieczorem?"))
            (progn
              (setf (npc-data os :oczekiwana-odpowiedz) "ciemno")
              (try 'powiedz os :dokogo *id-gracz* :cytat
                   "Nie mona tego zobaczy ani dotkn palcami, % Nie mona wyczu wchem ani usysze uszami; % Jest pod grami, jest nad gwiazdami, % Pustej jaskini nie omija, % Po nas zostanie, byo przed nami, % ycie gasi, a miech zabija."))))
      (when (eql (gdzie (get-label-id "ring") :stop :os) os)
        (try 'wyrzuc os :co (get-label-id "ring")))
      (let ((retv (or (try 'go-direction os :direction :SW)
                      (try 'go-direction os :direction :N)
                      (random-movement os))))
        (when (or retv (= 0 (random 5)))
          (setf (npc-data os :mumbling) nil
                (npc-data os :question) nil
                (npc-data os :take-ring) nil))
        retv)))

(defun fbard (os)
  (or (odwet os)
      (zalegle-polecenia os)
      (and (npc-data os :direction)
           (try 'go-direction os :direction (npc-data os :direction)))))




