
;;  "Hobbit" wersja polska
;;  Copyright (C) 2005 Pawe Aleksander Fedoryski <pfedor@fuw.edu.pl>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(defparameter *data-directory*
  (pathname-directory
   (concatenate 'string (directory-namestring *load-truename*) "/../dane/")))

(defparameter *types*
  '("gracz"
    "dzialania"
    "czasowniki"
    "rzeczowniki"
    "mapa"
    "postacie"
    "przedmioty"
    "drzwi"
    "przymiotniki"
    "sk_prz_drz"
    "sk_prz_os"
    "sk_prz_prz"
    "sk_cz_dz"))

(defvar *persistent-data* (make-hash-table))
(defvar *dictionary*)
(defvar *obiekty->przymiotniki*)
(defvar *czasowniki->dzialania*)
(defvar *labels* (make-hash-table :test #'equal))
(defvar *index-id*)

(defvar *data-remembered* nil)
(defvar *persistent-data-orig*)
(defvar *dictionary-orig*)
(defvar *obiekty->przymiotniki-orig*)
(defvar *czasowniki->dzialania-orig*)
(defvar *labels-orig*)

(defvar *last-id* 0)
(defvar *defaults*)

(defvar *id-gracz*)

(defun next-id ()
  (incf *last-id*))

(defun clear-id-seq ()
  (setf *last-id* 0))

(defun clear-labels ()
  (setf *labels* (make-hash-table :test #'equal)))

(defun clear-data ()
  (setf *persistent-data* (make-hash-table)))

(defun copy-hash (h)
  (let ((retv (make-hash-table :test (hash-table-test h))))
    (maphash #'(lambda (k v) (setf (gethash k retv) (copy-tree v)))
             h)
    retv))

(defun copy-vector (a)
  (map 'vector #'copy-tree a))

(defun remember-data ()
  (setf *persistent-data-orig* (copy-hash *persistent-data*)
        *labels-orig* (copy-hash *labels*)
        *dictionary-orig* (copy-hash *dictionary*)
        *obiekty->przymiotniki-orig* (copy-vector *obiekty->przymiotniki*)
        *czasowniki->dzialania-orig* (copy-vector *czasowniki->dzialania*)
        *data-remembered* t))

(defun restore-data ()
  (setf *persistent-data* (copy-hash *persistent-data-orig*)
        *labels* (copy-hash *labels-orig*)
        *dictionary* (copy-hash *dictionary-orig*)
        *obiekty->przymiotniki* (copy-vector *obiekty->przymiotniki-orig*)
        *czasowniki->dzialania* (copy-vector *czasowniki->dzialania-orig*))
  (create-index-id))

(defun set-label (label value)
  (setf (gethash label *labels*) value))

(defun get-label-id (label)
  (gethash label *labels*))

(defun read-elems (fn)
  (with-open-file (in fn)
    (loop with retv = () and elem = ""
          for c = (read-char in nil) do
           (when (or (eql c #\@) (null c))
            (push elem retv)
            (setf elem ""))
          (when (null c)
            (return (nreverse retv)))
          (unless (eql c #\@)
            (setf elem (concatenate 'string elem `(,c)))))))

(defun whitespacep (c)
  (not (char-not-equal c #\Space #\Newline #\Tab)))

(defun get-one-word (s &key (separator #'whitespacep))
  (let (pos1 pos2)
    (setf pos1 (position-if-not separator s))
    (when (null pos1)
      (return-from get-one-word (values nil s)))
    (setf pos2 (position-if separator s :start pos1))
    (values (subseq s pos1 pos2)
            (if (null pos2) "" (subseq s pos2)))))

(defun process-elem-label (elem)
  (let ((label (get-one-word elem)))
    (when (or (null label)
              (find-if-not #'alphanumericp label)
              (equal label "default"))
      (return-from process-elem-label))
    (when (gethash label *labels*)
      (return-from process-elem-label))
    (set-label label (next-id))))

(defun process-labels (els)
  (mapc #'process-elem-label els))

(defun colon-end-p (s)
  (eql (elt s (1- (count-if (constantly t) s))) #\:))

(defun process-value (s &optional (kw-p nil))
  (setf s (subseq s (position-if-not #'whitespacep s)))
  (cond
    ((eql (elt s 0) #\')
     (loop with esc = nil and v = ""
           for i from 1 do
           (cond (esc (setf esc nil))
                 ((eql (elt s i) #\\) (setf esc t))
                 ((eql (elt s i) #\')
                  (return (values v (subseq s (1+ i))))))
           (unless esc
             (setf v (concatenate 'string v (list (elt s i)))))))
    ((eql (elt s 0) #\*)
     (let (label)
       (setf (values label s) (get-one-word s))
       (setf label (subseq label 1))
       (when (null (get-label-id label))
         (format t "UWAGA: nierozpoznana etykieta ~s~%" label))
       (values (get-label-id label) s)))
    (t
     (multiple-value-bind (v s) (get-one-word s)
       (setf v (cond
                 ((every #'digit-char-p v) (parse-integer v))
                 ((or (equal v "T") (equal v "N")) (equal v "T"))
                 ((or kw-p
                      (find v '("m" "z" "n" "nmos" "mos" "poj" "mn"
                                "przedmioty" "postacie" "drzwi")
                            :test #'equal))
                  (intern (string-upcase v) "KEYWORD"))
                 (t v)))
       (values v s)))))

(defun process-fields (l s)
  (do ((k ":") v)
      ((or (null k)
           (not (colon-end-p k)))
       (values l s))
    (setf (values k s) (get-one-word s))
    (when (and k (colon-end-p k))
      (setf (values v s) (process-value s (search "->" k)))
      (setf (getf l (intern (string-upcase (remove #\: k)) "KEYWORD")) v))))

(defun process-defaults (s)
  (setf (values *defaults* s) (process-fields *defaults* s))
  s)

(defun add-defaults (l)
  (loop for k on *defaults* by #'cddr do
        (setf (getf l (car k)) (cadr k)))
  l)

(defun process-elem (elem)
  (let (word elem1 (retv ()))
    (setf (values word elem1) (get-one-word elem))
    (when (null word)
      (return-from process-elem retv))
    (when (equal word "default")
      (setf elem (process-defaults elem1))
      (setf (values word elem1) (get-one-word elem))
      (when (null word)
        (return-from process-elem retv)))
    (unless (colon-end-p word)
      (setf (getf retv :id) (get-label-id word))
      (setf elem elem1))
    (setf retv (add-defaults retv))
    (process-fields retv elem)))

(defun create-index-id ()
  (setf *index-id* (make-array (1+ *last-id*) :initial-element '()))
  (maphash #'(lambda (tt l)
               (dolist (o l)
                 (let ((id (getf o :id)))
                   (when id
                     (when (null (elt *index-id* id))
                       (setf (elt *index-id* id) (list tt)))
                     (push o (cdr (elt *index-id* id)))))))
           *persistent-data*))

(defun build-dictionary ()
  (setf *dictionary* (make-hash-table :test #'equalp))
  (loop for (tt kl id) in `((:rzeczowniki ,*przypadki* (:id :rodz))
                            (:przymiotniki ,*przypadki* (:id :rodz))
                            (:czasowniki (:rozkaz) (:id)))
        do
        (dolist (o (gethash tt *persistent-data*))
          (dolist (s (remove-duplicates (mapcar #'(lambda (p) (getf o p)) kl)
                                        :test #'equalp))
            (when s
              (push
               (cons (remove-if-not #'(lambda (p) (equalp s (getf o p))) kl)
                     (mapcan #'(lambda (k) (list k (getf o k))) id))
               (getf (gethash s *dictionary*) tt)))))))

(defun zapisz-skojarzenia ()
  (setf *obiekty->przymiotniki*
        (make-array (1+ *last-id*) :initial-element '()))
  (dolist (tt '((:sk_prz_drz :drzwi)
                (:sk_prz_os :osoba)
                (:sk_prz_prz :przedm)))
    (dolist (sk (gethash (car tt) *persistent-data*))
      (push (getf sk :przym)
            (elt *obiekty->przymiotniki* (getf sk (cadr tt))))))
  (setf *czasowniki->dzialania*
        (make-array (1+ *last-id*) :initial-element '()))
  (dolist (sk (gethash :sk_cz_dz *persistent-data*))
    (push (getf sk :dzial)
          (elt *czasowniki->dzialania* (getf sk :czas)))))

(defun read-data (types)
  (if *data-remembered*
      (restore-data)
      (progn
        (clear-id-seq)
        (clear-labels)
        (clear-data)
        (setf *id-gracz* 0)
        (set-label "gracz" *id-gracz*)
        (let ((hel (make-hash-table :test #'equal)))
          (dolist (tn types)
            (let ((filename (make-pathname :name tn :type "dane"
                                           :directory *data-directory*)))
              (setf (gethash tn hel) (read-elems filename))))
          (loop for v being the hash-values in hel do (process-labels v))
          (maphash
           #'(lambda (h v)
               (let ((*defaults* ()))
                 (setf (gethash (intern (string-upcase h) "KEYWORD")
                                *persistent-data*)
                       (remove nil (mapcar #'process-elem v)))))
           hel))
        (create-index-id)
        (build-dictionary)
        (zapisz-skojarzenia)
        (remember-data))))

(defun find-object (id &optional rodz)
  (find-if #'(lambda (o)
               (or (null rodz)
                   (eql (getf o :rodz) rodz)))
           (cdr (elt *index-id* id))))

(defun rodzaj (id)
  (if (eql id *id-gracz*)
      :m
      (let* ((rzid (getf (cadr (elt *index-id* id)) :nazwa))
             (lrz (cdr (elt *index-id* rzid))))
        (dolist (rodz '(:z :m :n :nmos :mos))
          (when (find rodz lrz :key #'(lambda (o) (getf o :rodz)))
            (return-from rodzaj rodz))))))

(defmacro zob (pole id &optional rodz)
  `(getf (find-object ,id ,rodz) ,pole))

(defun przymiotniki-opisujace (id)
  (elt *obiekty->przymiotniki* id))

(defun ust-przymiotniki-opisujace (id val)
  (setf (elt *obiekty->przymiotniki* id) val))

(defsetf przymiotniki-opisujace ust-przymiotniki-opisujace)

(defun przyp (przypadek id
              &key bez-przymiotnikow poczatek-zdania dluga-forma)
  (when (eql id *id-gracz*)
    (return-from przyp (if dluga-forma
                           (get-const *you-long* przypadek)
                           (get-const *you* przypadek))))
  (let ((rodz (rodzaj id))
        rz
        (lprz '()))
    (if (eql (kto-pierscien) id)
        (setf rz (get-const *someone* przypadek))
        (progn
          (setf rz (zob przypadek (zob :nazwa id) rodz))
          (setf lprz 
                (mapcar #'(lambda (przid) (zob przypadek przid rodz))
                        (przymiotniki-opisujace id)))))
    (when poczatek-zdania
      (if (and lprz (null bez-przymiotnikow))
          (setf (car lprz) (string-capitalize (car lprz)))
          (setf rz (string-capitalize rz))))
    (format nil "~@[~{~a ~}~]~a" (unless bez-przymiotnikow lprz) rz)))

(defmacro def-przyp (p)
  `(defun ,(intern (symbol-name p)) (&rest args)
     (push ,p args)
     (apply #'przyp args)))

(dolist (p *przypadki*)
  (eval `(def-przyp ,p)))

(defun matches (tt field value)
  (mapcar #'(lambda (o) (getf o :id))
          (remove-if-not #'(lambda (o) (eql (getf o field) value))
                         (gethash tt *persistent-data*))))

(defun postacie-w-lokacji (lokid)
  (matches :postacie :gdzie_lok lokid))

(defun przedmioty-w-lokacji (lokid)
  (matches :przedmioty :gdzie_lok lokid))

(defun przedmioty-wewnatrz (przid)
  (matches :przedmioty :gdzie_prz przid))

(defun postacie-wewnatrz (przid)
  (matches :postacie :gdzie_prz przid))

(defun przedmioty-trzyma (osid)
  (matches :przedmioty :gdzie_osoba osid))

(defun postacie-trzyma (osid)
  (matches :postacie :gdzie_osoba osid))

(defun lokacje-z-drzwiami (drzid)
  (mapcar #'(lambda (o) (getf o :id))
          (remove-if-not
           #'(lambda (lok)
               (find drzid (keys *kierdrzwi*)
                     :key #'(lambda (dd) (getf lok dd))))
           (gethash :mapa *persistent-data*))))

(defun gdzie (id &key stop)
  (let ((gdzie_lok (zob :gdzie_lok id))
        (gdzie_prz (zob :gdzie_prz id))
        (gdzie_osoba (zob :gdzie_osoba id)))
    (cond (gdzie_lok)
          ((and (find stop `(:prz :przedmiot ,gdzie_prz))
                gdzie_prz))
          ((and (find stop `(:os :osoba ,gdzie_osoba))
                gdzie_osoba))
          ((and gdzie_prz (not (zob :otwarte gdzie_prz)))
           gdzie_prz)
          (t (gdzie (or gdzie_prz gdzie_osoba) :stop stop)))))

(defun find-type (id)
  (car (elt *index-id* id)))

(defun ust-gdzie (id id-val)
  (let ((o (find-object id)))
    (remf o :gdzie_lok)
    (remf o :gdzie_prz)
    (remf o :gdzie_osoba))
  (let ((pole (getf '(:mapa :gdzie_lok
                      :przedmioty :gdzie_prz
                      :postacie :gdzie_osoba
                      :gracz :gdzie_osoba) (find-type id-val))))
    (setf (zob pole id) id-val))
  (when (eql id (get-label-id "ring"))
    (ust-kto-pierscien nil)))

(defsetf gdzie ust-gdzie)

(defun dictionary (k)
  (gethash k *dictionary*))

(defun istnieje-lpoj (id)
  (some #'(lambda (o) (find (getf o :rodz) '(:z :m :n)))
        (cdr (elt *index-id* id))))

(defun co-widzi (os)
  (let ((id (gdzie os)))
    (append
     (loop for i from 0 to *last-id*
        when (and (find (car (elt *index-id* i))
                        '(:postacie :przedmioty :gracz))
                  (eql (gdzie i) id)
                  (not (eql os i))
                  (not (eql (kto-pierscien) i))
                  (or (eql (kto-pierscien) os)
                      (not (eql (gdzie i :stop (kto-pierscien))
                                (kto-pierscien)))))
        collecting i)
     (when (eql (find-type id) :mapa)
       (remove nil (mapcar #'(lambda (k) (zob k id)) (keys *kierdrzwi*)))))))

(defun dzialania-opisywane (idcz &optional (c nil c-supplied-p))
  (let ((dzialania (elt *czasowniki->dzialania* idcz))
        (czycytat (not (null c))))
    (if c-supplied-p
        (remove-if-not #'(lambda (dz) (eql czycytat (zob :czycytat dz)))
                       dzialania)
        dzialania)))

(defun akceptrodz (id rodz)
  (or (and (= id *id-gracz*) (or (null rodz) (find rodz '(:m :mos))))
      (and (not (= id *id-gracz*))
           (not (null (find-object (zob :nazwa id) rodz))))))

(defun type-ids (tt)
  (loop for i from 0 to *last-id*
        when (eql tt (car (elt *index-id* i)))
        collecting i))
